from PySide6.QtCore import Qt, Signal, Slot
from PySide6.QtWidgets import (
    QVBoxLayout,
    QCheckBox,
)

from rare.models.game import RareGame
from rare.models.install import UninstallOptionsModel
from rare.utils.misc import qta_icon
from rare.widgets.dialogs import ButtonDialog, game_title


class UninstallDialog(ButtonDialog):
    result_ready = Signal(UninstallOptionsModel)

    def __init__(self, rgame: RareGame, options: UninstallOptionsModel, parent=None):
        super(UninstallDialog, self).__init__(parent=parent)
        header = self.tr("Uninstall")
        self.setWindowTitle(game_title(header, rgame.app_title))
        self.setSubtitle(game_title(header, rgame.app_title))

        self.keep_files = QCheckBox(self.tr("Keep files"))
        self.keep_files.setChecked(bool(options.keep_files))
        self.keep_files.setEnabled(not rgame.is_overlay)

        self.keep_folder = QCheckBox(self.tr("Keep game folder"))
        self.keep_folder.setChecked(bool(options.keep_folder))
        self.keep_folder.setEnabled(not rgame.is_overlay)

        self.keep_config = QCheckBox(self.tr("Keep configuation"))
        self.keep_config.setChecked(bool(options.keep_config))
        self.keep_config.setEnabled(not rgame.is_overlay)

        self.keep_overlay_keys = QCheckBox(self.tr("Keep EOS Overlay registry keys"))
        self.keep_overlay_keys.setChecked(bool(options.keep_overlay_keys))
        self.keep_overlay_keys.setEnabled(rgame.is_overlay)

        layout = QVBoxLayout()
        layout.addWidget(self.keep_files)
        layout.addWidget(self.keep_folder)
        layout.addWidget(self.keep_config)
        layout.addWidget(self.keep_overlay_keys)

        self.setCentralLayout(layout)

        self.accept_button.setText(self.tr("Uninstall"))
        self.accept_button.setIcon(qta_icon("ri.uninstall-line"))
        self.accept_button.setObjectName("UninstallButton")

        self.keep_files.checkStateChanged.connect(self.__on_keep_files_changed)

        self.options: UninstallOptionsModel = options

    @Slot(Qt.CheckState)
    def __on_keep_files_changed(self, state: Qt.CheckState):
        enabled = state != Qt.CheckState.Unchecked
        self.keep_folder.setCheckState(state if enabled else Qt.CheckState.Checked)
        self.keep_folder.setDisabled(enabled)

    def done_handler(self) -> None:
        self.result_ready.emit(self.options)

    def accept_handler(self):
        self.options.set_accepted(
            keep_files=self.keep_files.isChecked(),
            keep_folder=self.keep_folder.isChecked(),
            keep_config=self.keep_config.isChecked(),
            keep_overlay_keys=self.keep_overlay_keys.isChecked(),
        )

    def reject_handler(self):
        self.options.set_rejected()



from logging import getLogger
from typing import Callable, Tuple, Any

from PySide6.QtCore import Signal, QObject
from PySide6.QtWidgets import QApplication

from rare.components.tabs.store.constants import (
    wishlist_query,
    search_query,
    wishlist_add_query,
    wishlist_remove_query,
)
from rare.utils.paths import cache_dir
from rare.utils.qt_requests import QtRequests
from .api.models.query import SearchStoreQuery
from .api.models.diesel import DieselProduct
from .api.models.response import (
    ResponseModel,
    CatalogOfferModel,
)

logger = getLogger("StoreAPI")
graphql_url = "https://graphql.epicgames.com/graphql"


DEBUG: Callable[[], bool] = lambda: "--debug" in QApplication.arguments()


class StoreAPI(QObject):
    update_wishlist = Signal()

    def __init__(self, token, language: str, country: str, installed):
        super(StoreAPI, self).__init__()
        self.token = token
        self.language_code: str = language
        self.country_code: str = country
        self.locale = f"{self.language_code}-{self.country_code}"
        self.manager = QtRequests(parent=self)
        self.authed_manager = QtRequests(token=token, parent=self)
        self.cached_manager = QtRequests(cache=str(cache_dir().joinpath("store")), parent=self)

        self.installed = installed

        self.browse_active = False
        self.next_browse_request = tuple(())

    def get_free(self, callback: callable):
        url = "https://store-site-backend-static-ipv4.ak.epicgames.com/freeGamesPromotions"
        params = {
            "locale": self.locale,
            "country": self.country_code,
            "allowCountries": self.country_code,
        }
        self.manager.get(url, lambda data: self.__handle_free_games(data, callback), params=params)

    @staticmethod
    def __handle_free_games(data, callback):
        try:
            response = ResponseModel.from_dict(data)
            if response.errors:
                for error in response.errors:
                    logger.error(error)
            elements = response.data.catalog.searchStore.elements
        except (Exception, AttributeError, KeyError) as e:
            if DEBUG():
                raise e
            elements = False
            logger.error("Free games request failed with: %s", e)
        callback(elements)

    def get_wishlist(self, callback):
        self.authed_manager.post(
            graphql_url,
            lambda data: self.__handle_wishlist(data, callback),
            {
                "query": wishlist_query,
                "variables": {
                    "country": self.country_code,
                    "locale": self.locale,
                    "withPrice": True,
                },
            },
        )

    @staticmethod
    def __handle_wishlist(data, callback):
        try:
            response = ResponseModel.from_dict(data)
            if response.errors:
                for error in response.errors:
                    logger.error(error)
            elements = response.data.wishlist.wishlistItems.elements
        except (Exception, AttributeError, KeyError) as e:
            if DEBUG():
                raise e
            elements = False
            logger.error("Wishlist request failed with: %s", e)
        callback(elements)

    def search_game(self, name, callback):
        payload = {
            "query": search_query,
            "variables": {
                "category": "games/edition/base|bundles/games|editors|software/edition/base",
                "count": 20,
                "country": self.country_code,
                "keywords": name,
                "locale": self.locale,
                "sortDir": "DESC",
                "allowCountries": self.country_code,
                "start": 0,
                "tag": "",
                "withMapping": False,
                "withPrice": True,
            },
        }

        self.manager.post(graphql_url, lambda data: self.__handle_search(data, callback), payload)

    @staticmethod
    def __handle_search(data, callback):
        try:
            response = ResponseModel.from_dict(data)
            if response.errors:
                for error in response.errors:
                    logger.error(error)
            elements = response.data.catalog.searchStore.elements
        except (Exception, AttributeError, KeyError) as e:
            if DEBUG():
                raise e
            elements = False
            logger.error("Search request failed with: %s", e)
        callback(elements)

    def browse_games(self, browse_model: SearchStoreQuery, callback):
        if self.browse_active:
            self.next_browse_request = (browse_model, callback)
            return
        self.browse_active = True
        payload = {
            "query": search_query,
            "variables": browse_model.to_dict()
        }
        self.manager.post(graphql_url, lambda data: self.__handle_browse_games(data, callback), payload)

    def __handle_browse_games(self, data, callback):
        self.browse_active = False
        if data is None:
            data = {}
        if not self.next_browse_request:
            try:
                response = ResponseModel.from_dict(data)
                if response.errors:
                    for error in response.errors:
                        logger.error(error)
                elements = response.data.catalog.searchStore.elements
            except (Exception, AttributeError, KeyError) as e:
                if DEBUG():
                    raise e
                elements = False
                logger.error("Browse request failed with: %s", e)
            callback(elements)
        else:
            self.browse_games(*self.next_browse_request)  # pylint: disable=E1120
            self.next_browse_request = tuple(())

    # def get_game_config_graphql(self, namespace: str, callback):
    #     payload = {
    #         "query": config_query,
    #         "variables": {
    #             "namespace": namespace
    #         }
    #     }

    def __make_graphql_query(self):
        pass

    def __make_api_query(self):
        pass

    def get_game_config_cms(self, slug: str, is_bundle: bool, callback):
        url = "https://store-content.ak.epicgames.com/api"
        url += f"/{self.locale}/content/{'products' if not is_bundle else 'bundles'}/{slug}"
        self.manager.get(url, lambda data: self.__handle_get_game(data, callback))

    @staticmethod
    def __handle_get_game(data, callback):
        try:
            product = DieselProduct.from_dict(data)
            callback(product)
        except Exception as e:
            if DEBUG():
                raise e
            logger.error(str(e))
            # callback({})

    # needs a captcha
    def add_to_wishlist(self, namespace, offer_id, callback: callable):
        payload = {
            "query": wishlist_add_query,
            "variables": {
                "offerId": offer_id,
                "namespace": namespace,
                "country": self.country_code,
                "locale": self.locale,
            },
        }
        self.authed_manager.post(graphql_url, lambda data: self._handle_add_to_wishlist(data, callback), payload)

    def _handle_add_to_wishlist(self, data, callback):
        try:
            response = ResponseModel.from_dict(data)
            if response.errors:
                for error in response.errors:
                    logger.error(error)
            success = response.data.wishlist.addToWishlist.success
        except Exception as e:
            if DEBUG():
                raise e
            logger.error("Add to wishlist request failed with: %s", e)
            success = False
        callback(success)
        self.update_wishlist.emit()

    def remove_from_wishlist(self, namespace, offer_id, callback: callable):
        payload = {
            "query": wishlist_remove_query,
            "variables": {
                "offerId": offer_id,
                "namespace": namespace,
                "operation": "REMOVE",
            },
        }
        self.authed_manager.post(graphql_url, lambda data: self._handle_remove_from_wishlist(data, callback),
                                 payload)

    def _handle_remove_from_wishlist(self, data, callback):
        try:
            response = ResponseModel.from_dict(data)
            if response.errors:
                for error in response.errors:
                    logger.error(error)
            success = response.data.wishlist.removeFromWishlist.success
        except Exception as e:
            if DEBUG():
                raise e
            logger.error("Remove from wishlist request failed with: %s", e)
            success = False
        callback(success)
        self.update_wishlist.emit()



import logging
from datetime import datetime, timezone
from typing import List

from PySide6.QtCore import Qt, Slot, Signal, QObject, QEvent
from PySide6.QtGui import QShowEvent, QHideEvent, QResizeEvent
from PySide6.QtWidgets import (
    QHBoxLayout,
    QWidget,
    QSizePolicy,
    QVBoxLayout,
    QSpacerItem,
    QScrollArea,
    QFrame,
)

from rare.components.tabs.store.api.models.response import CatalogOfferModel, WishlistItemModel
from rare.widgets.flow_layout import FlowLayout
from rare.widgets.side_tab import SideTabContents
from rare.widgets.sliding_stack import SlidingStackedWidget
from .store_api import StoreAPI
from .widgets.details import StoreDetailsWidget
from .widgets.groups import StoreGroup
from .widgets.items import StoreItemWidget

logger = logging.getLogger("StoreLanding")


class LandingPage(SlidingStackedWidget, SideTabContents):

    def __init__(self, store_api: StoreAPI, parent=None):
        super(LandingPage, self).__init__(parent=parent)
        self.implements_scrollarea = True

        self.landing_widget = LandingWidget(store_api, parent=self)
        self.landing_widget.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        self.landing_widget.set_title.connect(self.set_title)
        self.landing_widget.show_details.connect(self.show_details)

        self.landing_scroll = QScrollArea(self)
        self.landing_scroll.setWidgetResizable(True)
        self.landing_scroll.setFrameStyle(QFrame.Shape.NoFrame | QFrame.Shadow.Plain)
        self.landing_scroll.setWidget(self.landing_widget)
        self.landing_scroll.widget().setAutoFillBackground(False)
        self.landing_scroll.viewport().setAutoFillBackground(False)

        self.details_widget = StoreDetailsWidget([], store_api, parent=self)
        self.details_widget.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        self.details_widget.set_title.connect(self.set_title)
        self.details_widget.back_clicked.connect(self.show_main)

        self.setDirection(Qt.Orientation.Horizontal)
        self.addWidget(self.landing_scroll)
        self.addWidget(self.details_widget)

    @Slot()
    def show_main(self):
        self.slideInWidget(self.landing_scroll)

    @Slot(object)
    def show_details(self, game: CatalogOfferModel):
        self.details_widget.update_game(game)
        self.slideInWidget(self.details_widget)


class FreeGamesScroll(QScrollArea):
    def __init__(self, parent=None):
        super(FreeGamesScroll, self).__init__(parent=parent)
        self.setObjectName(type(self).__name__)

    def setWidget(self, w):
        super().setWidget(w)
        w.installEventFilter(self)

    def eventFilter(self, a0: QObject, a1: QEvent) -> bool:
        if a0 is self.widget() and a1.type() == QEvent.Type.Resize:
            self.__resize(a0)
            return a0.event(a1)
        return False

    def __resize(self, e: QResizeEvent):
        minh = self.horizontalScrollBar().minimum()
        maxh = self.horizontalScrollBar().maximum()
        # lk: when the scrollbar is not visible, min and max are 0
        if maxh > minh:
            height = (
                e.size().height()
                + self.rect().height() // 2
                - self.contentsRect().height() // 2
                + self.widget().layout().spacing()
                + self.horizontalScrollBar().sizeHint().height()
            )
        else:
            height = e.size().height() + self.rect().height() - self.contentsRect().height()
        self.setMinimumHeight(max(height, self.minimumHeight()))


class LandingWidget(QWidget, SideTabContents):
    show_details = Signal(CatalogOfferModel)

    def __init__(self, api: StoreAPI, parent=None):
        super(LandingWidget, self).__init__(parent=parent)
        self.api = api

        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 3, 0)
        self.setLayout(layout)
        self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)

        self.free_games_now = StoreGroup(self.tr("Free now"), layout=QHBoxLayout, parent=self)
        self.free_games_now.main_layout.setAlignment(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter)
        self.free_games_now.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)

        self.free_games_next = StoreGroup(self.tr("Free next week"), layout=QHBoxLayout, parent=self)
        self.free_games_next.main_layout.setAlignment(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter)
        self.free_games_next.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)

        self.discounts_group = StoreGroup(self.tr("Wishlist discounts"), layout=FlowLayout, parent=self)
        self.discounts_group.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)

        self.games_group = StoreGroup(self.tr("Free to play"), FlowLayout, self)
        self.games_group.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        self.games_group.loading(False)
        self.games_group.setVisible(False)

        free_scroll = FreeGamesScroll(self)
        free_container = QWidget(free_scroll)
        free_scroll.setWidget(free_container)
        free_container_layout = QHBoxLayout(free_container)

        free_scroll.setWidgetResizable(True)
        free_scroll.setFrameShape(QScrollArea.Shape.NoFrame)
        free_scroll.setSizeAdjustPolicy(QScrollArea.SizeAdjustPolicy.AdjustToContents)
        free_scroll.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)

        free_container_layout.setContentsMargins(0, 0, 0, 0)
        free_container_layout.setAlignment(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignTop)
        free_container_layout.setSizeConstraint(QHBoxLayout.SizeConstraint.SetFixedSize)
        free_container_layout.addWidget(self.free_games_now)
        free_container_layout.addWidget(self.free_games_next)

        free_scroll.widget().setAutoFillBackground(False)
        free_scroll.viewport().setAutoFillBackground(False)

        # layout.addWidget(self.free_games_now, alignment=Qt.AlignmentFlag.AlignTop)
        # layout.addWidget(self.free_games_next, alignment=Qt.AlignmentFlag.AlignTop)
        layout.addWidget(free_scroll, alignment=Qt.AlignmentFlag.AlignTop)
        layout.addWidget(self.discounts_group, alignment=Qt.AlignmentFlag.AlignTop)
        layout.addWidget(self.games_group, alignment=Qt.AlignmentFlag.AlignTop)
        layout.addItem(QSpacerItem(0, 0, QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Expanding))

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous():
            return super().showEvent(a0)
        self.api.get_free(self.__update_free_games)
        self.api.get_wishlist(self.__update_wishlist_discounts)
        return super().showEvent(a0)

    def hideEvent(self, a0: QHideEvent) -> None:
        if a0.spontaneous():
            return super().hideEvent(a0)
        # TODO: Implement tab unloading
        return super().hideEvent(a0)

    def __update_wishlist_discounts(self, wishlist: List[WishlistItemModel]):
        for w in self.discounts_group.findChildren(StoreItemWidget, options=Qt.FindChildOption.FindDirectChildrenOnly):
            self.discounts_group.layout().removeWidget(w)
            w.deleteLater()

        for item in filter(lambda x: bool(x.offer.price.totalPrice.discount), wishlist):
            w = StoreItemWidget(self.api.cached_manager, item.offer)
            w.show_details.connect(self.show_details)
            self.discounts_group.layout().addWidget(w)
        have_discounts = any(map(lambda x: bool(x.offer.price.totalPrice.discount), wishlist))
        self.discounts_group.setVisible(have_discounts)
        self.discounts_group.loading(False)

    def __update_free_games(self, free_games: List[CatalogOfferModel]):
        for w in self.free_games_now.findChildren(StoreItemWidget, options=Qt.FindChildOption.FindDirectChildrenOnly):
            self.free_games_now.layout().removeWidget(w)
            w.deleteLater()

        for w in self.free_games_next.findChildren(StoreItemWidget, options=Qt.FindChildOption.FindDirectChildrenOnly):
            self.free_games_next.layout().removeWidget(w)
            w.deleteLater()

        date = datetime.now(timezone.utc)
        free_now = []
        free_next = []
        for item in free_games:
            try:
                if item.price.totalPrice.discountPrice == 0:
                    free_now.append(item)
                    continue
                if item.title == "Mystery Game":
                    free_next.append(item)
                    continue
            except KeyError as e:
                logger.warning(str(e))

            if item.promotions is not None:
                if not item.promotions.promotionalOffers:
                    start_date = item.promotions.upcomingPromotionalOffers[0].promotionalOffers[0].startDate
                else:
                    start_date = item.promotions.promotionalOffers[0].promotionalOffers[0].startDate

                if start_date > date:
                    free_next.append(item)

        # free games now
        self.free_games_now.setVisible(bool(free_now))
        for item in free_now:
            w = StoreItemWidget(self.api.cached_manager, item)
            w.show_details.connect(self.show_details)
            self.free_games_now.layout().addWidget(w)
        self.free_games_now.loading(False)

        # free games next week
        self.free_games_next.setVisible(bool(free_next))
        for item in free_next:
            w = StoreItemWidget(self.api.cached_manager, item)
            if item.title != "Mystery Game":
                w.show_details.connect(self.show_details)
            self.free_games_next.layout().addWidget(w)
        self.free_games_next.loading(False)

    def show_games(self, data):
        if not data:
            return

        for w in self.games_group.findChildren(StoreItemWidget, options=Qt.FindChildOption.FindDirectChildrenOnly):
            self.games_group.layout().removeWidget(w)
            w.deleteLater()

        for game in data:
            w = StoreItemWidget(self.api.cached_manager, game)
            w.show_details.connect(self.show_details)
            self.games_group.layout().addWidget(w)
        self.games_group.loading(False)



from PySide6.QtCore import Qt, Signal
from PySide6.QtWidgets import QStyle, QPushButton, QLineEdit

from rare.utils.misc import qta_icon


class ButtonLineEdit(QLineEdit):
    buttonClicked = Signal()

    def __init__(self, icon_name, placeholder_text: str, parent=None):
        super(ButtonLineEdit, self).__init__(parent=parent)
        self.setObjectName(type(self).__name__)

        self.button = QPushButton(self)
        self.button.setObjectName(f"{type(self).__name__}Button")
        self.button.setIcon(qta_icon(icon_name))
        self.button.setCursor(Qt.CursorShape.ArrowCursor)
        self.button.clicked.connect(self.buttonClicked.emit)

        self.setPlaceholderText(placeholder_text)
        # frame_width = self.style().pixelMetric(QStyle.PM_DefaultFrameWidth)
        # button_size = self.button.sizeHint()
        #
        # self.setStyleSheet(
        #     f"QLineEdit#{self.objectName()} {{padding-right: {(button_size.width() + frame_width + 1)}px; }}"
        # )
        # self.setMinimumSize(
        #     max(self.minimumSizeHint().width(), button_size.width() + frame_width * 2 + 2),
        #     max(
        #         self.minimumSizeHint().height(),
        #         button_size.height() + frame_width * 2 + 2,
        #     ),
        # )

    def resizeEvent(self, event):
        frame_width = self.style().pixelMetric(QStyle.PixelMetric.PM_DefaultFrameWidth)
        button_size = self.button.sizeHint()
        self.button.move(
            self.rect().right() - frame_width - button_size.width(),
            (self.rect().bottom() - button_size.height() + 1) // 2,
        )
        super(ButtonLineEdit, self).resizeEvent(event)



from abc import abstractmethod
from enum import IntEnum
from logging import getLogger
from typing import List, Dict, Tuple, Union, Optional

from PySide6.QtCore import Signal, Qt
from PySide6.QtGui import QIntValidator, QDoubleValidator, QShowEvent
from PySide6.QtWidgets import QGroupBox, QCheckBox, QLineEdit, QComboBox

from rare.ui.components.tabs.settings.widgets.overlay import Ui_OverlaySettings
from rare.utils import config_helper as config

logger = getLogger("GameOverlays")


class OverlayLineEdit(QLineEdit):
    def __init__(self, option: str, placeholder: str, parent=None):
        self.option = option
        super(OverlayLineEdit, self).__init__(parent=parent)
        self.valueChanged = self.textChanged

        self.setPlaceholderText(placeholder)

    def setDefault(self):
        self.setText("")

    def getValue(self) -> Optional[str]:
        return f"{self.option}={text}" if (text := self.text()) else None

    def setValue(self, options: Dict[str, str]):
        if (value := options.get(self.option, None)) is not None:
            self.setText(value)
            options.pop(self.option)
        else:
            self.setDefault()


class OverlayComboBox(QComboBox):
    def __init__(self, option: str, parent=None):
        self.option = option
        super(OverlayComboBox, self).__init__(parent=parent)
        self.valueChanged = self.currentIndexChanged

    def setDefault(self):
        self.setCurrentIndex(0)

    def getValue(self) -> Optional[str]:
        return f"{self.option}={self.currentData(Qt.ItemDataRole.UserRole)}" if self.currentIndex() > 0 else None

    def setValue(self, options: Dict[str, str]):
        if (value := options.get(self.option, None)) is not None:
            self.setCurrentIndex(self.findData(value, Qt.ItemDataRole.UserRole))
            options.pop(self.option)
        else:
            self.setDefault()


class OverlayCheckBox(QCheckBox):
    def __init__(self, option: str, title: str, desc: str = "", default_enabled: bool = False, parent=None):
        self.option = option
        super().__init__(title, parent=parent)
        self.setChecked(default_enabled)
        self.default_enabled = default_enabled
        self.setToolTip(desc)

    def setDefault(self):
        self.setChecked(self.default_enabled)

    def getValue(self) -> Optional[str]:
        # lk: return the check state in case of non-default, otherwise None
        checked = self.isChecked()
        value = f"{self.option}={int(checked)}" if self.default_enabled else self.option
        return value if checked ^ self.default_enabled else None

    def setValue(self, options: Dict[str, str]):
        if options.get(self.option, None) is not None:
            self.setChecked(not self.default_enabled)
            options.pop(self.option)
        else:
            self.setChecked(self.default_enabled)


class OverlayStringInput(OverlayLineEdit):
    def __init__(self, option: str, placeholder: str, parent=None):
        super().__init__(option, placeholder, parent=parent)


class OverlayNumberInput(OverlayLineEdit):
    def __init__(self, option: str, placeholder: Union[int, float], parent=None):
        super().__init__(option, str(placeholder), parent=parent)
        validator = QDoubleValidator(self) if isinstance(placeholder, float) else QIntValidator(self)
        self.setValidator(validator)


class OverlaySelectInput(OverlayComboBox):
    def __init__(self, option: str, values: Tuple, parent=None):
        super().__init__(option, parent=parent)
        for item in values:
            text, data = item
            self.addItem(text, data)
        # self.addItems([str(v) for v in values])
        # self.addItems(map(str, values))


class ActivationStates(IntEnum):
    GLOBAL = -1
    DISABLED = 0
    DEFAULTS = 1
    CUSTOM = 2


class OverlaySettings(QGroupBox):
    # str: option key
    environ_changed = Signal(str)

    def __init__(self, parent=None):
        super(OverlaySettings, self).__init__(parent=parent)
        self.ui = Ui_OverlaySettings()
        self.ui.setupUi(self)

        self.ui.show_overlay_combo.addItem(self.tr("Global"), ActivationStates.GLOBAL)
        self.ui.show_overlay_combo.addItem(self.tr("Disabled"), ActivationStates.DISABLED)
        self.ui.show_overlay_combo.addItem(self.tr("Enabled (defaults)"), ActivationStates.DEFAULTS)
        self.ui.show_overlay_combo.addItem(self.tr("Enabled (custom)"), ActivationStates.CUSTOM)

        self.envvar: str = None
        self.force_disabled: str = None
        self.force_defaults: str = None
        self.app_name: str = "default"

        self.option_widgets: List[Union[OverlayCheckBox, OverlayLineEdit, OverlayComboBox]] = []
        # self.checkboxes: Dict[str, OverlayCheckBox] = {}
        # self.values: Dict[str, Union[OverlayLineEdit, OverlayComboBox]] = {}

        self.ui.options_group.setTitle(self.tr("Custom options"))
        self.ui.show_overlay_combo.currentIndexChanged.connect(self.update_settings)

    def setupWidget(
        self,
        grid_map: List[OverlayCheckBox],
        form_map: List[Tuple[Union[OverlayLineEdit, OverlayComboBox], str]],
        envvar: str,
        force_disabled: str,
        force_defaults: str,
    ):
        self.envvar = envvar
        self.force_disabled = force_disabled
        self.force_defaults = force_defaults

        for i, widget in enumerate(grid_map):
            widget.setParent(self.ui.options_group)
            self.ui.options_grid.addWidget(widget, i // 4, i % 4)
            # self.checkboxes[widget.option] = widget
            self.option_widgets.append(widget)
            widget.stateChanged.connect(self.update_settings)

        for widget, label in form_map:
            widget.setParent(self.ui.options_group)
            self.ui.options_form.addRow(label, widget)
            # self.values[widget.option] = widget
            self.option_widgets.append(widget)
            widget.valueChanged.connect(self.update_settings)

    @abstractmethod
    def update_settings_override(self, state: ActivationStates):
        raise NotImplementedError

    def update_settings(self):
        current_state = self.ui.show_overlay_combo.currentData(Qt.ItemDataRole.UserRole)
        self.ui.options_group.setEnabled(current_state == ActivationStates.CUSTOM)

        if current_state == ActivationStates.GLOBAL:
            # System default (don't add any env variables)
            config.remove_envvar(self.app_name, self.envvar)

        elif current_state == ActivationStates.DISABLED:
            # hidden
            config.set_envvar(self.app_name, self.envvar, self.force_disabled)

        elif current_state == ActivationStates.DEFAULTS:
            config.set_envvar(self.app_name, self.envvar, self.force_defaults)

        elif current_state == ActivationStates.CUSTOM:
            self.ui.options_group.setDisabled(False)
            # custom options
            options = (name for widget in self.option_widgets if (name := widget.getValue()) is not None)

            config.set_envvar(self.app_name, self.envvar, ",".join(options))

        self.environ_changed.emit(self.envvar)
        self.update_settings_override(current_state)

    def setCurrentState(self, state: ActivationStates):
        self.ui.show_overlay_combo.setCurrentIndex(self.ui.show_overlay_combo.findData(state, Qt.ItemDataRole.UserRole))
        self.ui.options_group.setEnabled(state == ActivationStates.CUSTOM)

    def showEvent(self, a0: QShowEvent):
        if a0.spontaneous():
            return super().showEvent(a0)
        self.ui.options_group.blockSignals(True)

        config_options = config.get_envvar(self.app_name, self.envvar, fallback=None)
        if config_options is None:
            logger.debug("Overlay setting %s is not present", self.envvar)
            self.setCurrentState(ActivationStates.GLOBAL)

        elif config_options == self.force_disabled:
            self.setCurrentState(ActivationStates.DISABLED)

        elif config_options == self.force_defaults:
            self.setCurrentState(ActivationStates.DEFAULTS)

        else:
            self.setCurrentState(ActivationStates.CUSTOM)
            opts = {}
            for o in config_options.split(","):
                if "=" in o:
                    k, v = o.split("=")
                    opts[k] = v
                else:
                    # lk: The value doesn't matter other than not being None
                    opts[o] = "enable"

            for widget in self.option_widgets:
                widget.setValue(opts)
            if opts:
                logger.info("Remaining options without a gui switch: %s", ",".join(opts.keys()))

        self.ui.options_group.blockSignals(False)
        return super().showEvent(a0)


class DxvkSettings(OverlaySettings):
    def __init__(self, parent=None):
        super(DxvkSettings, self).__init__(parent=parent)
        self.setTitle(self.tr("DXVK settings"))
        grid = [
            OverlayCheckBox("fps", self.tr("FPS")),
            OverlayCheckBox("frametime", self.tr("Frametime")),
            OverlayCheckBox("memory", self.tr("Memory usage")),
            OverlayCheckBox("gpuload", self.tr("GPU usage")),
            OverlayCheckBox("devinfo", self.tr("Device info")),
            OverlayCheckBox("version", self.tr("DXVK version")),
            OverlayCheckBox("api", self.tr("D3D feature level")),
            OverlayCheckBox("compiler", self.tr("Compiler activity")),
        ]
        form = [
            (OverlayNumberInput("scale", 1.0), self.tr("Scale"))
        ]
        self.setupWidget(grid, form, "DXVK_HUD", "0", "1")

    def update_settings_override(self, state: ActivationStates):
        pass


mangohud_position = (
    ("default", "default"),
    ("top-left", "top-left"),
    ("top-right", "top-right"),
    ("middle-left", "middle-left"),
    ("middle-right", "middle-right"),
    ("bottom-left", "bottom-left"),
    ("bottom-right", "bottom-right"),
    ("top-center", "top-center"),
)

mangohud_vsync = (
    ("config", None),
    ("adaptive", "0"),
    ("off", "1"),
    ("mailbox", "2"),
    ("on", "3"),
)

mangohud_gl_vsync = (
    ("config", None),
    ("off", "0"),
    ("on", "1"),
    ("half", "2"),
    ("third", "3"),
    ("quarter", "4"),
)


class MangoHudSettings(OverlaySettings):
    def __init__(self, parent=None):
        super(MangoHudSettings, self).__init__(parent=parent)
        self.setTitle(self.tr("MangoHud settings"))
        grid = [
            OverlayCheckBox("read_cfg", self.tr("Read config")),
            OverlayCheckBox("fps", self.tr("FPS"), default_enabled=True),
            OverlayCheckBox("frame_timing", self.tr("Frame time"), default_enabled=True),
            OverlayCheckBox("cpu_stats", self.tr("CPU load"), default_enabled=True),
            OverlayCheckBox("gpu_stats", self.tr("GPU load"), default_enabled=True),
            OverlayCheckBox("cpu_temp", self.tr("CPU temperature")),
            OverlayCheckBox("gpu_temp", self.tr("GPU temperature")),
            OverlayCheckBox("ram", self.tr("Memory usage")),
            OverlayCheckBox("vram", self.tr("VRAM usage")),
            OverlayCheckBox("time", self.tr("Local time")),
            OverlayCheckBox("version", self.tr("MangoHud version")),
            OverlayCheckBox("arch", self.tr("System architecture")),
            OverlayCheckBox("histogram", self.tr("FPS graph")),
            OverlayCheckBox("gpu_name", self.tr("GPU name")),
            OverlayCheckBox("cpu_power", self.tr("CPU power consumption")),
            OverlayCheckBox("gpu_power", self.tr("GPU power consumption")),
        ]
        form = [
            (OverlayNumberInput("fps_limit", 0), self.tr("FPS limit")),
            (OverlaySelectInput("vsync", mangohud_vsync), self.tr("Vulkan vsync")),
            (OverlaySelectInput("gl_vsync", mangohud_gl_vsync), self.tr("OpenGL vsync")),
            (OverlayNumberInput("font_size", 24), self.tr("Font size")),
            (OverlaySelectInput("position", mangohud_position), self.tr("Position")),
        ]

        self.setupWidget(grid, form, "MANGOHUD_CONFIG", "no_display", "read_cfg")

    def showEvent(self, a0: QShowEvent):
        if a0.spontaneous():
            return super().showEvent(a0)
        self.ui.options_group.blockSignals(True)
        self.ui.options_group.blockSignals(False)
        return super().showEvent(a0)

    def update_settings_override(self, state: IntEnum):  # pylint: disable=E0202
        if state == ActivationStates.GLOBAL:
            config.remove_envvar(self.app_name, "MANGOHUD")

        elif state == ActivationStates.DISABLED:
            config.set_envvar(self.app_name, "MANGOHUD", "0")

        elif state == ActivationStates.DEFAULTS:
            config.set_envvar(self.app_name, "MANGOHUD", "1")

        elif state == ActivationStates.CUSTOM:
            config.set_envvar(self.app_name, "MANGOHUD", "1")

        self.environ_changed.emit("MANGOHUD")


if __name__ == "__main__":
    import sys
    from PySide6.QtWidgets import QApplication, QDialog, QVBoxLayout

    from legendary.core import LegendaryCore

    core = LegendaryCore()
    config.init_config_handler(core)

    app = QApplication(sys.argv)
    dlg = QDialog()

    dxvk = DxvkSettings(dlg)
    mangohud = MangoHudSettings(dlg)

    layout = QVBoxLayout(dlg)
    layout.addWidget(dxvk)
    layout.addWidget(mangohud)

    dlg.show()
    ret = app.exec()
    config.save_config()
    sys.exit(ret)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'eos_widget.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFormLayout, QGroupBox, QHBoxLayout,
    QLabel, QLayout, QPushButton, QSizePolicy,
    QStackedWidget, QVBoxLayout, QWidget)

class Ui_EosWidget(object):
    def setupUi(self, EosWidget):
        if not EosWidget.objectName():
            EosWidget.setObjectName(u"EosWidget")
        EosWidget.resize(465, 98)
        EosWidget.setWindowTitle(u"GroupBox")
        self.eos_layout = QVBoxLayout(EosWidget)
        self.eos_layout.setObjectName(u"eos_layout")
        self.eos_layout.setSizeConstraint(QLayout.SetDefaultConstraint)
        self.overlay_stack = QStackedWidget(EosWidget)
        self.overlay_stack.setObjectName(u"overlay_stack")
        self.install_page = QWidget()
        self.install_page.setObjectName(u"install_page")
        self.install_page_layout = QHBoxLayout(self.install_page)
        self.install_page_layout.setObjectName(u"install_page_layout")
        self.install_page_layout.setContentsMargins(0, 0, 0, 0)
        self.install_label_layout = QFormLayout()
        self.install_label_layout.setObjectName(u"install_label_layout")
        self.install_label_layout.setLabelAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignmentFlag.AlignTop|Qt.AlignTrailing)
        self.install_label_layout.setFormAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignTop)
        self.install_label = QLabel(self.install_page)
        self.install_label.setObjectName(u"install_label")

        self.install_label_layout.setWidget(0, QFormLayout.LabelRole, self.install_label)

        self.install_text = QLabel(self.install_page)
        self.install_text.setObjectName(u"install_text")

        self.install_label_layout.setWidget(0, QFormLayout.ItemRole.FieldRole, self.install_text)


        self.install_page_layout.addLayout(self.install_label_layout)

        self.install_button_layout = QVBoxLayout()
        self.install_button_layout.setObjectName(u"install_button_layout")
        self.install_button_layout.setContentsMargins(-1, -1, 0, -1)
        self.install_button = QPushButton(self.install_page)
        self.install_button.setObjectName(u"install_button")
        self.install_button.setMinimumSize(QSize(140, 0))

        self.install_button_layout.addWidget(self.install_button, 0, Qt.AlignmentFlag.AlignTop)


        self.install_page_layout.addLayout(self.install_button_layout)

        self.install_page_layout.setStretch(0, 1)
        self.overlay_stack.addWidget(self.install_page)
        self.info_page = QWidget()
        self.info_page.setObjectName(u"info_page")
        self.info_page_layout = QHBoxLayout(self.info_page)
        self.info_page_layout.setObjectName(u"info_page_layout")
        self.info_page_layout.setContentsMargins(0, 0, 0, 0)
        self.info_label_layout = QFormLayout()
        self.info_label_layout.setObjectName(u"info_label_layout")
        self.info_label_layout.setLabelAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignmentFlag.AlignTop|Qt.AlignTrailing)
        self.info_label_layout.setFormAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignTop)
        self.version_label = QLabel(self.info_page)
        self.version_label.setObjectName(u"version_label")
        self.version_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.info_label_layout.setWidget(0, QFormLayout.LabelRole, self.version_label)

        self.path_label = QLabel(self.info_page)
        self.path_label.setObjectName(u"path_label")
        self.path_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.info_label_layout.setWidget(1, QFormLayout.LabelRole, self.path_label)


        self.info_page_layout.addLayout(self.info_label_layout)

        self.info_button_layout = QVBoxLayout()
        self.info_button_layout.setObjectName(u"info_button_layout")
        self.update_button = QPushButton(self.info_page)
        self.update_button.setObjectName(u"update_button")
        self.update_button.setMinimumSize(QSize(140, 0))

        self.info_button_layout.addWidget(self.update_button, 0, Qt.AlignmentFlag.AlignTop)

        self.uninstall_button = QPushButton(self.info_page)
        self.uninstall_button.setObjectName(u"uninstall_button")
        self.uninstall_button.setMinimumSize(QSize(140, 0))

        self.info_button_layout.addWidget(self.uninstall_button, 0, Qt.AlignmentFlag.AlignTop)


        self.info_page_layout.addLayout(self.info_button_layout)

        self.info_page_layout.setStretch(0, 1)
        self.overlay_stack.addWidget(self.info_page)

        self.eos_layout.addWidget(self.overlay_stack)


        self.retranslateUi(EosWidget)

        self.overlay_stack.setCurrentIndex(0)

    # setupUi

    def retranslateUi(self, EosWidget):
        EosWidget.setTitle(QCoreApplication.translate("EosWidget", u"Epic Overlay", None))
        self.install_label.setText(QCoreApplication.translate("EosWidget", u"Version", None))
        self.install_text.setText(QCoreApplication.translate("EosWidget", u"Epic Online Services Overlay is not installed", None))
        self.install_button.setText(QCoreApplication.translate("EosWidget", u"Install", None))
        self.version_label.setText(QCoreApplication.translate("EosWidget", u"Version", None))
        self.path_label.setText(QCoreApplication.translate("EosWidget", u"Install folder", None))
        self.update_button.setText(QCoreApplication.translate("EosWidget", u"Update", None))
        self.uninstall_button.setText(QCoreApplication.translate("EosWidget", u"Uninstall", None))
    # retranslateUi




import platform
import shutil
from logging import getLogger
from typing import Tuple

from PySide6.QtCore import QObject, Signal
from legendary.core import LegendaryCore
from legendary.lfs.eos import remove_registry_entries

from rare.lgndr.cli import LegendaryCLI
from rare.lgndr.glue.arguments import LgndrUninstallGameArgs
from rare.lgndr.glue.monkeys import LgndrIndirectStatus
from rare.models.game import RareGame
from rare.models.install import UninstallOptionsModel
from rare.utils import config_helper as config
from rare.utils.paths import desktop_links_supported, desktop_link_types, desktop_link_path
from .worker import Worker

logger = getLogger("UninstallWorker")


# TODO: You can use RareGame directly here once this is called inside RareCore and skip metadata fetch
def uninstall_game(
    core: LegendaryCore, rgame: RareGame, keep_files=False, keep_folder=True, keep_config=False, keep_overlay_keys=False
) -> Tuple[bool, str]:
    if rgame.is_overlay:
        logger.info('Deleting overlay installation...')
        core.remove_overlay_install()

        if keep_overlay_keys:
            return True, ""

        logger.info('Removing registry entries...')
        if platform.system() != "Window":
            prefixes = config.get_prefixes()
            if platform.system() == "Darwin":
                # TODO: add crossover support
                pass
            if len(prefixes):
                for prefix, _ in prefixes:
                    remove_registry_entries(prefix)
                    logger.debug("Removed registry entries for prefix %s", prefix)
        else:
            remove_registry_entries()

        return True, ""

    # remove shortcuts link
    if desktop_links_supported():
        for link_type in desktop_link_types():
            link_path = desktop_link_path(
                rgame.game.metadata.get("customAttributes", {}).get("FolderName", {}).get("value"), link_type
            )
            if link_path.exists():
                link_path.unlink(missing_ok=True)

    install_path = rgame.igame.install_path

    status = LgndrIndirectStatus()
    LegendaryCLI(core).uninstall_game(
        LgndrUninstallGameArgs(
            app_name=rgame.app_name,
            keep_files=keep_files,
            skip_uninstaller=False,
            yes=True,
            indirect_status=status,
        )
    )

    keep_folder = keep_files if keep_files else keep_folder
    if not keep_folder:
        logger.info("Removing game install directory")
        shutil.rmtree(install_path, ignore_errors=True)

    if not keep_config:
        logger.info("Removing sections in config file")
        config.remove_section(rgame.app_name)
        config.remove_section(f"{rgame.app_name}.env")

        config.save_config()

    return status.success, status.message


class UninstallWorker(Worker):
    class Signals(QObject):
        result = Signal(RareGame, bool, str)

    def __init__(self, core: LegendaryCore, rgame: RareGame, options: UninstallOptionsModel):
        super(UninstallWorker, self).__init__()
        self.signals = UninstallWorker.Signals()
        self.core = core
        self.rgame = rgame
        self.options = options

    def run_real(self) -> None:
        self.rgame.state = RareGame.State.UNINSTALLING
        success, message = uninstall_game(
            self.core,
            self.rgame,
            keep_files=self.options.keep_files,
            keep_folder=self.options.keep_folder,
            keep_config=self.options.keep_config,
            keep_overlay_keys=self.options.keep_overlay_keys,
        )
        self.rgame.state = RareGame.State.IDLE
        self.signals.result.emit(self.rgame, success, message)



from PySide6.QtCore import QObject


# Class to use QObject.tr()
class Constants(QObject):
    def __init__(self):
        super(Constants, self).__init__()
        self.categories = sorted(
            [
                (self.tr("Action"), "1216"),
                (self.tr("Adventure"), "1117"),
                (self.tr("Puzzle"), "1298"),
                (self.tr("Open world"), "1307"),
                (self.tr("Racing"), "1212"),
                (self.tr("RPG"), "1367"),
                (self.tr("Shooter"), "1210"),
                (self.tr("Strategy"), "1115"),
                (self.tr("Survival"), "1080"),
                (self.tr("First Person"), "1294"),
                (self.tr("Indie"), "1263"),
                (self.tr("Simulation"), "1393"),
                (self.tr("Sport"), "1283"),
            ],
            key=lambda x: x[0],
        )

        self.platforms = [
            ("MacOS", "9548"),
            ("Windows", "9547"),
        ]
        self.others = [
            (self.tr("Single player"), "1370"),
            (self.tr("Multiplayer"), "1203"),
            (self.tr("Controller"), "9549"),
            (self.tr("Co-op"), "1264"),
        ]

        self.types = [
            (self.tr("Editor"), "editors"),
            (self.tr("Game"), "games/edition/base"),
            (self.tr("Bundle"), "bundles/games"),
            (self.tr("Add-on"), "addons"),
            (self.tr("Apps"), "software/edition/base"),
        ]


__Image = '''
type
url
alt
'''

__StorePageMapping = '''
cmsSlug
offerId
prePurchaseOfferId
'''

__PageSandboxModel = '''
pageSlug
pageType
productId
sandboxId
createdDate
updatedDate
deletedDate
mappings {
  %s
}
''' % (__StorePageMapping)

__CatalogNamespace = '''
parent
displayName
store
home: mappings(pageType: "productHome") {
  %s
}
addons: mappings(pageType: "addon--cms-hybrid") {
  %s
}
offers: mappings(pageType: "offer") {
  %s
}
''' % (__PageSandboxModel, __PageSandboxModel, __PageSandboxModel)

__CatalogItem = '''
id
namespace
'''

__GetPriceRes = '''
  totalPrice {
    discountPrice
    originalPrice
    voucherDiscount
    discount
    currencyCode
    currencyInfo {
      decimals
      symbol
    }
    fmtPrice(locale: $locale) {
      originalPrice
      discountPrice
      intermediatePrice
    }
  }
  lineOffers {
    appliedRules {
      id
      endDate
      discountSetting {
        discountType
      }
    }
  }
'''

__Promotions = '''
promotionalOffers {
    promotionalOffers {
        startDate
        endDate
        discountSetting {
            discountType
            discountPercentage
        }
    }
}
upcomingPromotionalOffers {
    promotionalOffers {
        startDate
        endDate
        discountSetting {
            discountType
            discountPercentage
        }
    }
}
'''

__CatalogOffer = '''
title
id
namespace
offerType
expiryDate
status
isCodeRedemptionOnly
description
effectiveDate
keyImages {
    %(image)s
}
currentPrice
seller {
  id
  name
}
productSlug
urlSlug
url
tags {
  id
  name
  groupName
}
items {
    %(catalog_item)s
}
customAttributes {
  key
  value
}
categories {
  path
}
catalogNs @include(if: $withMapping) {
    %(catalog_namespace)s
}
offerMappings @include(if: $withMapping) {
    %(page_sandbox_model)s
}
price(country: $country) @include(if: $withPrice) {
    %(get_price_res)s
}
promotions(category: $category) @include(if: $withPromotions) {
    %(promotions)s
}
''' % {
    "image": __Image,
    "catalog_item": __CatalogItem,
    "catalog_namespace": __CatalogNamespace,
    "page_sandbox_model": __PageSandboxModel,
    "get_price_res": __GetPriceRes,
    "promotions": __Promotions,
}

__Pagination = '''
count
total
'''

SEARCH_STORE_QUERY = '''
query searchStoreQuery(
  $allowCountries: String
  $category: String
  $count: Int
  $country: String!
  $keywords: String
  $locale: String
  $namespace: String
  $withMapping: Boolean = false
  $itemNs: String
  $sortBy: String
  $sortDir: String
  $start: Int
  $tag: String
  $releaseDate: String
  $withPrice: Boolean = false
  $withPromotions: Boolean = false
  $priceRange: String
  $freeGame: Boolean
  $onSale: Boolean
  $effectiveDate: String
) {
  Catalog {
    searchStore(
      allowCountries: $allowCountries
      category: $category
      count: $count
      country: $country
      keywords: $keywords
      locale: $locale
      namespace: $namespace
      itemNs: $itemNs
      sortBy: $sortBy
      sortDir: $sortDir
      releaseDate: $releaseDate
      start: $start
      tag: $tag
      priceRange: $priceRange
      freeGame: $freeGame
      onSale: $onSale
      effectiveDate: $effectiveDate
    ) {
      elements {
        %s
      }
      paging {
        %s
      }
    }
  }
}
''' % (__CatalogOffer, __Pagination)

__WISHLIST_ITEM = '''
id
order
created
offerId
updated
namespace
isFirstTime
offer(locale: $locale) {
  %s
}
''' % __CatalogOffer

WISHLIST_QUERY = '''
query wishlistQuery(
  $country: String!
  $locale: String
  $category: String
  $withMapping: Boolean = false
  $withPrice: Boolean = false
  $withPromotions: Boolean = false
) {
  Wishlist {
    wishlistItems {
      elements {
        %s
      }
    }
  }
}
''' % __WISHLIST_ITEM

WISHLIST_ADD_QUERY = '''
mutation addWishlistMutation(
  $namespace: String!
  $offerId: String!
  $country: String!
  $locale: String
  $category: String
  $withMapping: Boolean = false
  $withPrice: Boolean = false
  $withPromotions: Boolean = false
) {
  Wishlist {
    addToWishlist(
      namespace: $namespace
      offerId: $offerId
    ) {
      wishlistItem {
        %s
      }
      success
    }
  }
}
''' % __WISHLIST_ITEM

WISHLIST_REMOVE_QUERY = '''
mutation removeFromWishlistMutation(
  $namespace: String!
  $offerId: String!
  $operation: RemoveOperation!
) {
  Wishlist {
    removeFromWishlist(
      namespace: $namespace
      offerId: $offerId
      operation: $operation
    ) {
      success
    }
  }
}
'''

COUPONS_QUERY = '''
query getCoupons(
  $currencyCountry: String!
  $identityId: String!
  $locale: String
) {
  CodeRedemption {
    coupons(
      currencyCountry: $currencyCountry
      identityId: $identityId
      includeSalesEventInfo: true
    ) {
      code
      codeStatus
      codeType
      consumptionMetadata {
        amountDisplay {
          amount
          currency
          placement
          symbol
        }
        minSalesPriceDisplay {
          amount
          currency
          placement
          symbol
        }
      }
      endDate
      namespace
      salesEvent(locale: $locale) {
        eventName
        eventSlug
        voucherImages {
          type
          url
        }
        voucherLink
      }
      startDate
    }
  }
}
'''

STORE_CONFIG_QUERY = '''
query getStoreConfig(
  $includeCriticReviews: Boolean = false
  $locale: String!
  $sandboxId: String!
  $templateId: String
) {
  Product {
    sandbox(sandboxId: $sandboxId) {
      configuration(locale: $locale, templateId: $templateId) {
        ... on StoreConfiguration {
          configs {
            shortDescription
            criticReviews @include(if: $includeCriticReviews) {
              openCritic
            }
            socialLinks {
              platform
              url
            }
            supportedAudio
            supportedText
            tags(locale: $locale) {
              id
              name
              groupName
            }
            technicalRequirements {
              macos {
                minimum
                recommended
                title
              }
              windows {
                minimum
                recommended
                title
              }
            }
          }
        }
        ... on HomeConfiguration {
          configs {
            keyImages {
              ... on KeyImage {
                type
                url
                alt
              }
            }
            longDescription
          }
        }
      }
    }
  }
}
'''


def compress_query(query: str) -> str:
    return query.replace("  ", "").replace("\n", " ")


game_query = compress_query(SEARCH_STORE_QUERY)
search_query = compress_query(SEARCH_STORE_QUERY)
wishlist_query = compress_query(WISHLIST_QUERY)
wishlist_add_query = compress_query(WISHLIST_ADD_QUERY)
wishlist_remove_query = compress_query(WISHLIST_REMOVE_QUERY)
coupons_query = compress_query(COUPONS_QUERY)
store_config_query = compress_query(STORE_CONFIG_QUERY)


if __name__ == "__main__":
    print(SEARCH_STORE_QUERY)



from rare.shared import ArgumentsSingleton
from rare.widgets.side_tab import SideTabWidget
from .about import About
from .debug import DebugSettings
from .settings import GameSettings
from .legendary import LegendarySettings
from .rare import RareSettings


class SettingsTab(SideTabWidget):
    def __init__(self, parent=None):
        super(SettingsTab, self).__init__(parent=parent)
        self.args = ArgumentsSingleton()

        rare_settings = RareSettings(self)
        self.rare_index = self.addTab(rare_settings, "Rare")

        legendary_settings = LegendarySettings(self)
        self.legendary_index = self.addTab(legendary_settings, "Legendary")

        game_settings = GameSettings(self)
        self.settings_index = self.addTab(game_settings, self.tr("Defaults"))

        self.about = About(self)
        title = self.tr("About")
        self.about_index = self.addTab(self.about, title, title)
        self.about.update_available_ready.connect(
            lambda: self.tabBar().setTabText(self.about_index, "About (!)")
        )

        if self.args.debug:
            title = self.tr("Debug")
            self.debug_index = self.addTab(DebugSettings(self), title, title)

        self.setCurrentIndex(self.rare_index)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'wishlist_widget.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QHBoxLayout, QLabel, QPushButton,
    QSizePolicy, QVBoxLayout, QWidget)

class Ui_WishlistWidget(object):
    def setupUi(self, WishlistWidget):
        if not WishlistWidget.objectName():
            WishlistWidget.setObjectName(u"WishlistWidget")
        WishlistWidget.resize(202, 94)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(WishlistWidget.sizePolicy().hasHeightForWidth())
        WishlistWidget.setSizePolicy(sizePolicy)
        WishlistWidget.setWindowTitle(u"WishlistWIdget")
        self.main_layout = QHBoxLayout(WishlistWidget)
        self.main_layout.setObjectName(u"main_layout")
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.info_widget = QWidget(WishlistWidget)
        self.info_widget.setObjectName(u"info_widget")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.info_widget.sizePolicy().hasHeightForWidth())
        self.info_widget.setSizePolicy(sizePolicy1)
        self.verticalLayout_2 = QVBoxLayout(self.info_widget)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.title_label = QLabel(self.info_widget)
        self.title_label.setObjectName(u"title_label")
        font = QFont()
        font.setPointSize(16)
        self.title_label.setFont(font)
        self.title_label.setText(u"TextLabel")
        self.title_label.setWordWrap(True)

        self.verticalLayout_2.addWidget(self.title_label)

        self.developer = QLabel(self.info_widget)
        self.developer.setObjectName(u"developer")
        font1 = QFont()
        font1.setPointSize(12)
        self.developer.setFont(font1)
        self.developer.setText(u"TextLabel")

        self.verticalLayout_2.addWidget(self.developer)

        self.price_layout = QHBoxLayout()
        self.price_layout.setObjectName(u"price_layout")
        self.discount_price = QLabel(self.info_widget)
        self.discount_price.setObjectName(u"discount_price")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Preferred)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.discount_price.sizePolicy().hasHeightForWidth())
        self.discount_price.setSizePolicy(sizePolicy2)
        self.discount_price.setText(u"TextLabel")

        self.price_layout.addWidget(self.discount_price)

        self.price = QLabel(self.info_widget)
        self.price.setObjectName(u"price")
        self.price.setText(u"TextLabel")

        self.price_layout.addWidget(self.price)


        self.verticalLayout_2.addLayout(self.price_layout)


        self.main_layout.addWidget(self.info_widget, 0, Qt.AlignmentFlag.AlignTop)

        self.delete_button = QPushButton(WishlistWidget)
        self.delete_button.setObjectName(u"delete_button")
        sizePolicy3 = QSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Maximum)
        sizePolicy3.setHorizontalStretch(0)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.delete_button.sizePolicy().hasHeightForWidth())
        self.delete_button.setSizePolicy(sizePolicy3)

        self.main_layout.addWidget(self.delete_button)


        self.retranslateUi(WishlistWidget)
    # setupUi

    def retranslateUi(self, WishlistWidget):
        self.delete_button.setText("")
        pass
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'install_dialog.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QFormLayout,
    QFrame, QLabel, QSizePolicy, QVBoxLayout,
    QWidget)

class Ui_InstallDialog(object):
    def setupUi(self, InstallDialog):
        if not InstallDialog.objectName():
            InstallDialog.setObjectName(u"InstallDialog")
        InstallDialog.resize(197, 195)
        InstallDialog.setWindowTitle(u"InstallDialog")
        self.main_layout = QFormLayout(InstallDialog)
        self.main_layout.setObjectName(u"main_layout")
        self.main_layout.setLabelAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)
        self.install_dir_label = QLabel(InstallDialog)
        self.install_dir_label.setObjectName(u"install_dir_label")

        self.main_layout.setWidget(0, QFormLayout.LabelRole, self.install_dir_label)

        self.platform_label = QLabel(InstallDialog)
        self.platform_label.setObjectName(u"platform_label")

        self.main_layout.setWidget(1, QFormLayout.LabelRole, self.platform_label)

        self.platform_combo = QComboBox(InstallDialog)
        self.platform_combo.setObjectName(u"platform_combo")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.platform_combo.sizePolicy().hasHeightForWidth())
        self.platform_combo.setSizePolicy(sizePolicy)

        self.main_layout.setWidget(1, QFormLayout.ItemRole.FieldRole, self.platform_combo)

        self.shortcut_label = QLabel(InstallDialog)
        self.shortcut_label.setObjectName(u"shortcut_label")
        self.shortcut_label.setAlignment(Qt.AlignmentFlag.AlignLeading|Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignVCenter)

        self.main_layout.setWidget(2, QFormLayout.LabelRole, self.shortcut_label)

        self.shortcut_check = QCheckBox(InstallDialog)
        self.shortcut_check.setObjectName(u"shortcut_check")
        self.shortcut_check.setText(u"")

        self.main_layout.setWidget(2, QFormLayout.ItemRole.FieldRole, self.shortcut_check)

        self.selectable_layout = QVBoxLayout()
        self.selectable_layout.setObjectName(u"selectable_layout")

        self.main_layout.setLayout(3, QFormLayout.SpanningRole, self.selectable_layout)

        self.advanced_layout = QVBoxLayout()
        self.advanced_layout.setObjectName(u"advanced_layout")

        self.main_layout.setLayout(4, QFormLayout.SpanningRole, self.advanced_layout)

        self.download_size_label = QLabel(InstallDialog)
        self.download_size_label.setObjectName(u"download_size_label")

        self.main_layout.setWidget(5, QFormLayout.LabelRole, self.download_size_label)

        self.download_size_text = QLabel(InstallDialog)
        self.download_size_text.setObjectName(u"download_size_text")
        font = QFont()
        font.setItalic(True)
        self.download_size_text.setFont(font)

        self.main_layout.setWidget(5, QFormLayout.ItemRole.FieldRole, self.download_size_text)

        self.install_size_label = QLabel(InstallDialog)
        self.install_size_label.setObjectName(u"install_size_label")

        self.main_layout.setWidget(6, QFormLayout.LabelRole, self.install_size_label)

        self.install_size_text = QLabel(InstallDialog)
        self.install_size_text.setObjectName(u"install_size_text")
        self.install_size_text.setFont(font)
        self.install_size_text.setWordWrap(True)

        self.main_layout.setWidget(6, QFormLayout.ItemRole.FieldRole, self.install_size_text)

        self.avail_space_label = QLabel(InstallDialog)
        self.avail_space_label.setObjectName(u"avail_space_label")

        self.main_layout.setWidget(7, QFormLayout.LabelRole, self.avail_space_label)

        self.avail_space = QLabel(InstallDialog)
        self.avail_space.setObjectName(u"avail_space")
        font1 = QFont()
        font1.setBold(True)
        self.avail_space.setFont(font1)

        self.main_layout.setWidget(7, QFormLayout.ItemRole.FieldRole, self.avail_space)

        self.warning_label = QLabel(InstallDialog)
        self.warning_label.setObjectName(u"warning_label")

        self.main_layout.setWidget(8, QFormLayout.LabelRole, self.warning_label)

        self.warning_text = QLabel(InstallDialog)
        self.warning_text.setObjectName(u"warning_text")
        self.warning_text.setFont(font)
        self.warning_text.setFrameShape(QFrame.Shape.StyledPanel)
        self.warning_text.setFrameShadow(QFrame.Sunken)
        self.warning_text.setWordWrap(True)
        self.warning_text.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)

        self.main_layout.setWidget(8, QFormLayout.ItemRole.FieldRole, self.warning_text)


        self.retranslateUi(InstallDialog)
    # setupUi

    def retranslateUi(self, InstallDialog):
        self.install_dir_label.setText(QCoreApplication.translate("InstallDialog", u"Install folder", None))
        self.platform_label.setText(QCoreApplication.translate("InstallDialog", u"Platform", None))
        self.shortcut_label.setText(QCoreApplication.translate("InstallDialog", u"Create shortcut", None))
        self.download_size_label.setText(QCoreApplication.translate("InstallDialog", u"Download size", None))
        self.download_size_text.setText(QCoreApplication.translate("InstallDialog", u"Click verify...", None))
        self.install_size_label.setText(QCoreApplication.translate("InstallDialog", u"Total install size", None))
        self.install_size_text.setText(QCoreApplication.translate("InstallDialog", u"Click verify...", None))
        self.avail_space_label.setText(QCoreApplication.translate("InstallDialog", u"Available space", None))
        self.avail_space.setText("")
        self.warning_label.setText(QCoreApplication.translate("InstallDialog", u"Warning", None))
        self.warning_text.setText(QCoreApplication.translate("InstallDialog", u"None", None))
        pass
    # retranslateUi




import logging
from typing import List

from PySide6.QtCore import Qt, Signal, Slot
from PySide6.QtWidgets import (
    QCheckBox,
    QWidget,
    QSizePolicy,
    QScrollArea,
    QLabel,
)

from rare.ui.components.tabs.store.search import Ui_SearchWidget
from rare.widgets.button_edit import ButtonLineEdit
from rare.widgets.flow_layout import FlowLayout
from rare.widgets.side_tab import SideTabContents
from rare.widgets.sliding_stack import SlidingStackedWidget
from .api.models.query import SearchStoreQuery
from .api.models.response import CatalogOfferModel
from .constants import Constants
from .store_api import StoreAPI
from .widgets.details import StoreDetailsWidget
from .widgets.items import SearchItemWidget

logger = logging.getLogger("Shop")


class SearchPage(SlidingStackedWidget, SideTabContents):
    def __init__(self, store_api: StoreAPI, parent=None):
        super(SearchPage, self).__init__(parent=parent)
        self.implements_scrollarea = True

        self.search_widget = SearchWidget(store_api, parent=self)
        self.search_widget.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
        self.search_widget.set_title.connect(self.set_title)
        self.search_widget.show_details.connect(self.show_details)

        self.details_widget = StoreDetailsWidget([], store_api, parent=self)
        self.details_widget.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
        self.details_widget.set_title.connect(self.set_title)
        self.details_widget.back_clicked.connect(self.show_main)

        self.setDirection(Qt.Orientation.Horizontal)
        self.addWidget(self.search_widget)
        self.addWidget(self.details_widget)

    @Slot()
    def show_main(self):
        self.slideInWidget(self.search_widget)

    @Slot(object)
    def show_details(self, game: CatalogOfferModel):
        self.details_widget.update_game(game)
        self.slideInWidget(self.details_widget)


# noinspection PyAttributeOutsideInit,PyBroadException
class SearchWidget(QWidget, SideTabContents):
    show_details = Signal(CatalogOfferModel)

    def __init__(self, store_api: StoreAPI, parent=None):
        super(SearchWidget, self).__init__(parent=parent)
        self.implements_scrollarea = True
        self.ui = Ui_SearchWidget()
        self.ui.setupUi(self)
        self.ui.main_layout.setContentsMargins(0, 0, 3, 0)

        self.ui.filter_scrollarea.widget().setAutoFillBackground(False)
        self.ui.filter_scrollarea.viewport().setAutoFillBackground(False)

        self.store_api = store_api
        self.price = ""
        self.tags = []
        self.types = []
        self.update_games_allowed = True

        self.active_search_request = False
        self.next_search = ""
        self.wishlist: List = []

        self.search_bar = ButtonLineEdit("fa.search", placeholder_text=self.tr("Search"))
        self.results_scrollarea = ResultsWidget(self.store_api, self)
        self.results_scrollarea.show_details.connect(self.show_details)

        self.ui.left_layout.addWidget(self.search_bar)
        self.ui.left_layout.addWidget(self.results_scrollarea)

        self.search_bar.returnPressed.connect(self.show_search_results)
        self.search_bar.buttonClicked.connect(self.show_search_results)

        # self.init_filter()

    def load(self):
        # load browse games
        self.prepare_request()

    def show_search_results(self):
        if text := self.search_bar.text():
            self.results_scrollarea.load_results(text)
            # self.show_info.emit(self.search_bar.text())

    def init_filter(self):
        self.ui.none_price.toggled.connect(
            lambda: self.prepare_request("") if self.ui.none_price.isChecked() else None
        )
        self.ui.free_button.toggled.connect(
            lambda: self.prepare_request("free") if self.ui.free_button.isChecked() else None
        )
        self.ui.under10.toggled.connect(
            lambda: self.prepare_request("<price>[0, 1000)") if self.ui.under10.isChecked() else None
        )
        self.ui.under20.toggled.connect(
            lambda: self.prepare_request("<price>[0, 2000)") if self.ui.under20.isChecked() else None
        )
        self.ui.under30.toggled.connect(
            lambda: self.prepare_request("<price>[0, 3000)") if self.ui.under30.isChecked() else None
        )
        self.ui.above.toggled.connect(
            lambda: self.prepare_request("<price>[1499,]") if self.ui.above.isChecked() else None
        )
        # self.on_discount.toggled.connect(
        #     lambda: self.prepare_request("sale") if self.on_discount.isChecked() else None
        # )
        self.ui.on_discount.toggled.connect(lambda: self.prepare_request())
        constants = Constants()

        self.checkboxes = []

        for groupbox, variables in [
            (self.ui.genre_group, constants.categories),
            (self.ui.platform_group, constants.platforms),
            (self.ui.others_group, constants.others),
            (self.ui.type_group, constants.types),
        ]:
            for text, tag in variables:
                checkbox = CheckBox(text, tag)
                checkbox.activated.connect(lambda x: self.prepare_request(added_tag=x))
                checkbox.deactivated.connect(lambda x: self.prepare_request(removed_tag=x))
                groupbox.layout().addWidget(checkbox)
                self.checkboxes.append(checkbox)
        self.ui.reset_button.clicked.connect(self.reset_filters)
        self.ui.filter_scrollarea.setMinimumWidth(
            self.ui.filter_container.sizeHint().width()
            + self.ui.filter_container.layout().contentsMargins().left()
            + self.ui.filter_container.layout().contentsMargins().right()
            + self.ui.filter_scrollarea.verticalScrollBar().sizeHint().width()
        )

    def reset_filters(self):
        self.update_games_allowed = False
        for cb in self.checkboxes:
            cb.setChecked(False)
        self.ui.none_price.setChecked(True)

        self.tags = []
        self.types = []
        self.update_games_allowed = True

        self.ui.on_discount.setChecked(False)

    def prepare_request(
        self,
        price: str = None,
        added_tag: int = 0,
        removed_tag: int = 0,
        added_type: str = "",
        removed_type: str = "",
    ):
        if not self.update_games_allowed:
            return
        if price is not None:
            self.price = price

        if added_tag != 0:
            self.tags.append(added_tag)
        if removed_tag != 0 and removed_tag in self.tags:
            self.tags.remove(removed_tag)

        if added_type:
            self.types.append(added_type)
        if removed_type and removed_type in self.types:
            self.types.remove(removed_type)
        if (self.types or self.price) or self.tags or self.ui.on_discount.isChecked():
            # self.free_scrollarea.setVisible(False)
            self.discounts_group.setVisible(False)
        else:
            # self.free_scrollarea.setVisible(True)
            if len(self.discounts_group.layout().children()) > 0:
                self.discounts_group.setVisible(True)

        self.games_group.loading(True)

        browse_model = SearchStoreQuery(
            language=self.store_api.language_code,
            country=self.store_api.country_code,
            count=20,
            price_range=self.price,
            on_sale=self.ui.on_discount.isChecked(),
        )
        browse_model.tag = "|".join(self.tags)

        if self.types:
            browse_model.category = "|".join(self.types)
        self.store_api.browse_games(browse_model, self.show_games)


class CheckBox(QCheckBox):
    activated = Signal(str)
    deactivated = Signal(str)

    def __init__(self, text, tag):
        super(CheckBox, self).__init__(text)
        self.tag = tag

        self.toggled.connect(self.handle_toggle)

    def handle_toggle(self):
        if self.isChecked():
            self.activated.emit(self.tag)
        else:
            self.deactivated.emit(self.tag)


class ResultsWidget(QScrollArea):
    show_details = Signal(CatalogOfferModel)

    def __init__(self, store_api, parent=None):
        super(ResultsWidget, self).__init__(parent=parent)
        self.store_api = store_api

        self.results_container = QWidget(self)
        self.results_container.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
        self.results_layout = FlowLayout(self.results_container)
        self.setWidget(self.results_container)
        self.setWidgetResizable(True)

        # self.main_layout = QVBoxLayout(self)
        # self.main_layout.setContentsMargins(0, 0, 0, 0)
        # self.main_layout.addWidget(self.results_scrollarea)

        self.setEnabled(False)

    def load_results(self, text: str):
        self.setEnabled(False)
        if text != "":
            self.store_api.search_game(text, self.show_results)

    def show_results(self, results: dict):
        for w in self.results_container.findChildren(QLabel, options=Qt.FindChildOption.FindDirectChildrenOnly):
            self.results_layout.removeWidget(w)
            w.deleteLater()
        for w in self.results_container.findChildren(SearchItemWidget, options=Qt.FindChildOption.FindDirectChildrenOnly):
            self.results_layout.removeWidget(w)
            w.deleteLater()

        if not results:
            self.results_layout.addWidget(QLabel(self.tr("No results found")))
        else:
            for res in results:
                w = SearchItemWidget(self.store_api.cached_manager, res, parent=self.results_container)
                w.show_details.connect(self.show_details.emit)
                self.results_layout.addWidget(w)
        self.results_layout.update()
        self.setEnabled(True)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'queue_base_widget.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFrame, QHBoxLayout, QPushButton,
    QSizePolicy, QVBoxLayout, QWidget)

class Ui_QueueBaseWidget(object):
    def setupUi(self, QueueBaseWidget):
        if not QueueBaseWidget.objectName():
            QueueBaseWidget.setObjectName(u"QueueBaseWidget")
        QueueBaseWidget.resize(380, 62)
        QueueBaseWidget.setWindowTitle(u"QueueBaseWidget")
        QueueBaseWidget.setFrameShape(QFrame.Shape.StyledPanel)
        self.queue_base_layout = QHBoxLayout(QueueBaseWidget)
        self.queue_base_layout.setObjectName(u"queue_base_layout")
        self.queue_base_layout.setContentsMargins(0, 0, 0, 0)
        self.info_layout = QHBoxLayout()
        self.info_layout.setObjectName(u"info_layout")

        self.queue_base_layout.addLayout(self.info_layout)

        self.move_buttons = QWidget(QueueBaseWidget)
        self.move_buttons.setObjectName(u"move_buttons")
        self.move_buttons_layout = QVBoxLayout(self.move_buttons)
        self.move_buttons_layout.setObjectName(u"move_buttons_layout")
        self.move_buttons_layout.setContentsMargins(0, 0, 0, 0)
        self.move_up_button = QPushButton(self.move_buttons)
        self.move_up_button.setObjectName(u"move_up_button")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.move_up_button.sizePolicy().hasHeightForWidth())
        self.move_up_button.setSizePolicy(sizePolicy)
        self.move_up_button.setText(u"")
        self.move_up_button.setProperty("isSquare", 1)

        self.move_buttons_layout.addWidget(self.move_up_button, 0, Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignTop)

        self.move_down_button = QPushButton(self.move_buttons)
        self.move_down_button.setObjectName(u"move_down_button")
        sizePolicy.setHeightForWidth(self.move_down_button.sizePolicy().hasHeightForWidth())
        self.move_down_button.setSizePolicy(sizePolicy)
        self.move_down_button.setText(u"")
        self.move_down_button.setProperty("isSquare", 1)

        self.move_buttons_layout.addWidget(self.move_down_button, 0, Qt.AlignmentFlag.AlignLeft|Qt.AlignmentFlag.AlignBottom)


        self.queue_base_layout.addWidget(self.move_buttons)

        self.queue_buttons = QWidget(QueueBaseWidget)
        self.queue_buttons.setObjectName(u"queue_buttons")
        self.queue_buttons_layout = QVBoxLayout(self.queue_buttons)
        self.queue_buttons_layout.setObjectName(u"queue_buttons_layout")
        self.queue_buttons_layout.setContentsMargins(0, 0, 0, 0)
        self.force_button = QPushButton(self.queue_buttons)
        self.force_button.setObjectName(u"force_button")

        self.queue_buttons_layout.addWidget(self.force_button)

        self.remove_button = QPushButton(self.queue_buttons)
        self.remove_button.setObjectName(u"remove_button")

        self.queue_buttons_layout.addWidget(self.remove_button, 0, Qt.AlignmentFlag.AlignBottom)


        self.queue_base_layout.addWidget(self.queue_buttons)

        self.update_buttons = QWidget(QueueBaseWidget)
        self.update_buttons.setObjectName(u"update_buttons")
        self.update_buttons_layout = QVBoxLayout(self.update_buttons)
        self.update_buttons_layout.setObjectName(u"update_buttons_layout")
        self.update_buttons_layout.setContentsMargins(0, 0, 0, 0)
        self.update_button = QPushButton(self.update_buttons)
        self.update_button.setObjectName(u"update_button")

        self.update_buttons_layout.addWidget(self.update_button)

        self.settings_button = QPushButton(self.update_buttons)
        self.settings_button.setObjectName(u"settings_button")

        self.update_buttons_layout.addWidget(self.settings_button, 0, Qt.AlignmentFlag.AlignBottom)


        self.queue_base_layout.addWidget(self.update_buttons)

        self.queue_base_layout.setStretch(0, 1)

        self.retranslateUi(QueueBaseWidget)
    # setupUi

    def retranslateUi(self, QueueBaseWidget):
        self.force_button.setText(QCoreApplication.translate("QueueBaseWidget", u"Download now", None))
        self.remove_button.setText(QCoreApplication.translate("QueueBaseWidget", u"Remove from queue", None))
        self.update_button.setText(QCoreApplication.translate("QueueBaseWidget", u"Update game", None))
        self.settings_button.setText(QCoreApplication.translate("QueueBaseWidget", u"Update with settings", None))
        pass
    # retranslateUi




import datetime
import platform
from logging import getLogger
from typing import Union, Optional

from PySide6.QtCore import Signal, QSettings, Slot, QThreadPool, Qt
from PySide6.QtGui import QPixmap
from PySide6.QtWidgets import (
    QWidget,
    QMessageBox, QScrollArea, QVBoxLayout, QSizePolicy,
)

from rare.components.dialogs.install_dialog import InstallDialog
from rare.components.dialogs.uninstall_dialog import UninstallDialog
from rare.lgndr.models.downloading import UIUpdate
from rare.models.game import RareGame
from rare.models.image import ImageSize
from rare.models.install import InstallOptionsModel, InstallQueueItemModel, UninstallOptionsModel
from rare.models.options import options
from rare.shared import RareCore
from rare.shared.workers.install_info import InstallInfoWorker
from rare.shared.workers.uninstall import UninstallWorker
from rare.utils.misc import format_size
from rare.utils.paths import create_desktop_link, desktop_links_supported
from .download import DownloadWidget
from .groups import UpdateGroup, QueueGroup
from .thread import DlThread, DlResultModel, DlResultCode
from .widgets import UpdateWidget, QueueWidget

logger = getLogger("Download")


def get_time(seconds: Union[int, float]) -> str:
    return str(datetime.timedelta(seconds=seconds))


class DownloadsTab(QWidget):
    # int: number of updates
    update_title = Signal(int)

    def __init__(self, parent=None):
        super(DownloadsTab, self).__init__(parent=parent)
        self.rcore = RareCore.instance()
        self.core = RareCore.instance().core()
        self.signals = RareCore.instance().signals()
        self.args = RareCore.instance().args()

        self.__thread: Optional[DlThread] = None

        layout = QVBoxLayout(self)

        self.download_widget = DownloadWidget(self)
        self.download_widget.ui.kill_button.clicked.connect(self.stop_download)
        layout.addWidget(self.download_widget)

        self.queue_scrollarea = QScrollArea(self)
        self.queue_scrollarea.setWidgetResizable(True)
        self.queue_scrollarea.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.queue_scrollarea.setFrameStyle(QScrollArea.Shadow.Plain | QScrollArea.Shape.NoFrame)
        layout.addWidget(self.queue_scrollarea)

        queue_contents = QWidget(self.queue_scrollarea)
        queue_contents.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
        self.queue_scrollarea.setWidget(queue_contents)
        self.queue_scrollarea.widget().setAutoFillBackground(False)
        self.queue_scrollarea.viewport().setAutoFillBackground(False)

        queue_contents_layout = QVBoxLayout(queue_contents)
        queue_contents_layout.setContentsMargins(0, 0, 3, 0)

        self.queue_group = QueueGroup(self)
        self.queue_group.update_count.connect(self.update_queues_count)
        self.queue_group.removed.connect(self.__on_queue_removed)
        self.queue_group.force.connect(self.__on_queue_force)
        queue_contents_layout.addWidget(self.queue_group)

        self.updates_group = UpdateGroup(self)
        self.updates_group.update_count.connect(self.update_queues_count)
        self.updates_group.enqueue.connect(self.__get_install_options)
        queue_contents_layout.addWidget(self.updates_group)

        self.__check_updates()
        self.__reset_download()

        self.signals.game.install.connect(self.__get_install_options)
        self.signals.game.uninstall.connect(self.__get_uninstall_options)
        self.signals.download.enqueue.connect(self.__add_update)
        self.signals.download.dequeue.connect(self.__remove_update)

        self.__forced_item: Optional[InstallQueueItemModel] = None
        self.__omit_requeue = False

    @Slot()
    @Slot(int)
    def update_queues_count(self):
        count = self.updates_group.count() + self.queue_group.count() + (1 if self.is_download_active else 0)
        self.update_title.emit(count)

    @property
    def is_download_active(self):
        return self.__thread is not None

    def __check_updates(self):
        for rgame in self.rcore.updates:
            self.__add_update(rgame)

    @Slot(str)
    @Slot(RareGame)
    def __add_update(self, update: Union[str, RareGame]):
        if isinstance(update, str):
            update = self.rcore.get_game(update)

        auto_update = QSettings(self).value(
            f"{update.app_name}/{options.auto_update.key}",
            QSettings(self).value(*options.auto_update),
            options.auto_update.dtype
        )
        if auto_update:
            self.__get_install_options(
                InstallOptionsModel(app_name=update.app_name, update=True, silent=True)
            )
        else:
            self.updates_group.append(update.game, update.igame)
        self.update_queues_count()

    @Slot(str)
    def __remove_update(self, app_name):
        if self.__thread and self.__thread.item.options.app_name == app_name:
            self.stop_download(omit_queue=True)
        if self.queue_group.contains(app_name):
            self.queue_group.remove(app_name)
        if self.updates_group.contains(app_name):
            self.updates_group.remove(app_name)

    @Slot(str)
    def __on_queue_removed(self, app_name: str):
        """
        Handle removing a queued item.
        If the item exists in the updates (it means a repair was removed), re-enable the buttons.
        If it doesn't exist in the updates, recreate the widget.
        :param app_name:
        :return:
        """
        rgame = self.rcore.get_game(app_name)
        rgame.state = RareGame.State.IDLE
        if self.updates_group.contains(app_name):
            self.updates_group.set_widget_enabled(app_name, True)
        else:
            if rgame.is_installed and rgame.has_update:
                self.__add_update(app_name)

    @Slot(InstallQueueItemModel)
    def __on_queue_force(self, item: InstallQueueItemModel):
        if self.__thread:
            self.stop_download()
            self.__forced_item = item
        else:
            self.__start_download(item)

    def stop_download(self, omit_queue=False):
        """
        Stops the active download, by optionally skipping the queue

        :param omit_queue: bool
            If `True`, the stopped download won't be added back to the queue.
            Defaults to `False`
        :return:
        """
        self.__thread.kill()
        self.download_widget.ui.kill_button.setEnabled(False)
        # lk: if we are exiting Rare, wait for thread to finish
        # `self.on_exit` control whether we try to add the download
        # back in the queue. If we are on exit we wait for the thread
        # to finish, we do not care about handling the result really
        self.__omit_requeue = omit_queue
        if omit_queue:
            self.__thread.wait()

    def __refresh_download(self, item: InstallQueueItemModel):
        worker = InstallInfoWorker(self.core, item.options)
        worker.signals.result.connect(
            lambda d: self.__start_download(InstallQueueItemModel(options=item.options, download=d))
        )
        worker.signals.failed.connect(
            lambda m: logger.error(f"Failed to refresh download for {item.options.app_name} with error: {m}")
        )
        worker.signals.finished.connect(
            lambda: logger.info(f"Download refresh worker finished for {item.options.app_name}")
        )
        QThreadPool.globalInstance().start(worker)

    def __start_download(self, item: InstallQueueItemModel):
        rgame = self.rcore.get_game(item.options.app_name)
        if not rgame.state == RareGame.State.DOWNLOADING:
            logger.error(
                f"Can't start download {item.options.app_name}"
                f"due to incompatible state {RareGame.State(rgame.state).name}"
            )
            # lk: invalidate the queue item in case the game was uninstalled
            self.__requeue_download(InstallQueueItemModel(options=item.options))
            return
        if item.expired:
            self.__refresh_download(item)
            return
        dl_thread = DlThread(item, rgame, self.core, self.args.debug)
        dl_thread.result.connect(self.__on_download_result)
        dl_thread.progress.connect(self.__on_download_progress)
        dl_thread.finished.connect(dl_thread.deleteLater)
        dl_thread.start()
        self.__thread = dl_thread
        self.download_widget.ui.kill_button.setDisabled(False)
        self.download_widget.ui.dl_name.setText(item.download.game.app_title)
        self.download_widget.setPixmap(self.rcore.image_manager().get_pixmap(
            rgame.app_name, ImageSize.Wide, True
        ))

        self.signals.application.notify.emit(
            self.tr("Downloads"),
            self.tr("Starting: \"{}\" is now downloading.").format(rgame.app_title)
        )

    @Slot(UIUpdate, object)
    def __on_download_progress(self, ui_update: UIUpdate, dl_size: int):
        self.download_widget.ui.progress_bar.setValue(int(ui_update.progress))
        self.download_widget.ui.dl_speed.setText(f"{format_size(ui_update.download_compressed_speed)}/s")
        self.download_widget.ui.cache_used.setText(
            f"{format_size(ui_update.cache_usage) if ui_update.cache_usage > 1023 else '0KB'}"
        )
        self.download_widget.ui.downloaded.setText(
            f"{format_size(ui_update.total_downloaded)} / {format_size(dl_size)}"
        )
        self.download_widget.ui.time_left.setText(get_time(ui_update.estimated_time_left))

    def __requeue_download(self, item: InstallQueueItemModel):
        rgame = self.rcore.get_game(item.options.app_name)
        rgame.state = RareGame.State.DOWNLOADING
        self.queue_group.push_front(item, rgame.igame)
        logger.info(f"Re-queued download for {rgame.app_name} ({rgame.app_title})")

    @Slot(DlResultModel)
    def __on_download_result(self, result: DlResultModel):
        if result.code == DlResultCode.FINISHED:
            logger.info(f"Download finished: {result.options.app_name}")
            if result.shortcut and desktop_links_supported():
                if not create_desktop_link(
                    app_name=result.options.app_name,
                    app_title=result.app_title,
                    link_name=result.folder_name,
                    link_type="desktop",
                ):
                    # maybe add it to download summary, to show in finished downloads
                    logger.error(f"Failed to create desktop link on {platform.system()}")
                else:
                    logger.info(f"Created desktop link {result.folder_name} for {result.app_title}")

            self.signals.application.notify.emit(
                self.tr("Downloads"),
                self.tr("Finished: \"{}\" is now playable.").format(result.app_title),
            )

            if self.updates_group.contains(result.options.app_name):
                self.updates_group.set_widget_enabled(result.options.app_name, True)

        elif result.code == DlResultCode.ERROR:
            logger.error(f"Download error: {result.options.app_name} ({result.message})")
            QMessageBox.warning(self, self.tr("Error"), self.tr("Download error: {}").format(result.message))

        elif result.code == DlResultCode.STOPPED:
            logger.info(f"Download stopped: {result.options.app_name}")
            if not self.__omit_requeue:
                self.__requeue_download(InstallQueueItemModel(options=result.options))
            else:
                return

        # lk: if we finished a repair, and we have a disabled update, re-enable it
        if self.updates_group.contains(result.options.app_name):
            self.updates_group.set_widget_enabled(result.options.app_name, True)

        if result.code == DlResultCode.FINISHED and self.queue_group.count():
            self.__start_download(self.queue_group.pop_front())
        elif result.code == DlResultCode.STOPPED and self.__forced_item:
            self.__start_download(self.__forced_item)
            self.__forced_item = None
        else:
            self.__reset_download()

    def __reset_download(self):
        self.download_widget.setPixmap(QPixmap())
        self.download_widget.ui.kill_button.setDisabled(True)
        self.download_widget.ui.dl_name.setText(self.tr("No active download"))
        self.download_widget.ui.progress_bar.setValue(0)
        self.download_widget.ui.dl_speed.setText("...")
        self.download_widget.ui.time_left.setText("...")
        self.download_widget.ui.cache_used.setText("...")
        self.download_widget.ui.downloaded.setText("...")
        self.__thread = None

    @Slot(InstallOptionsModel)
    def __get_install_options(self, options: InstallOptionsModel):
        rgame = self.rcore.get_game(options.app_name)
        rgame.state = RareGame.State.DOWNLOADING
        install_dialog = InstallDialog(
            rgame,
            options=options,
            parent=self,
        )
        install_dialog.result_ready.connect(self.__on_install_dialog_closed)
        install_dialog.execute()

    @Slot(InstallQueueItemModel)
    def __on_install_dialog_closed(self, item: InstallQueueItemModel):
        rgame = self.rcore.get_game(item.options.app_name)
        if item and not item.download.game.is_dlc and not item.download.analysis.dl_size:
            rgame.set_installed(True)
            rgame.state = RareGame.State.IDLE
            return
        if item:
            # lk: start update only if there is no other active thread and there is no queue
            if self.__thread is None and not self.queue_group.count():
                self.__start_download(item)
            else:
                rgame = self.rcore.get_game(item.options.app_name)
                self.queue_group.push_back(item, rgame.igame)
            # lk: Handle repairing into the current version
            # When we add something to the queue from repair, we might select to update or not
            # if we do select to update with repair, we can remove the widget from the updates groups
            # otherwise we disable it and keep it in the updates
            if self.updates_group.contains(item.options.app_name):
                if item.download.igame.version == self.updates_group.get_widget_version(item.options.app_name):
                    self.updates_group.remove(item.options.app_name)
                else:
                    self.updates_group.set_widget_enabled(item.options.app_name, False)
        else:
            if self.updates_group.contains(item.options.app_name):
                self.updates_group.set_widget_enabled(item.options.app_name, True)
            rgame.state = RareGame.State.IDLE
        self.update_queues_count()

    @Slot(UninstallOptionsModel)
    def __get_uninstall_options(self, options: UninstallOptionsModel):
        rgame = self.rcore.get_game(options.app_name)
        rgame.state = RareGame.State.UNINSTALLING
        uninstall_dialog = UninstallDialog(
            rgame,
            options=options,
            parent=self,
        )
        uninstall_dialog.result_ready.connect(self.__on_uninstall_dialog_closed)
        uninstall_dialog.open()

    @Slot(UninstallOptionsModel)
    def __on_uninstall_dialog_closed(self, options: UninstallOptionsModel):
        rgame = self.rcore.get_game(options.app_name)
        if options and options.accepted:
            worker = UninstallWorker(self.core, rgame, options)
            worker.signals.result.connect(self.__on_uninstall_worker_result)
            QThreadPool.globalInstance().start(worker)
        else:
            rgame.state = RareGame.State.IDLE

    @Slot(RareGame, bool, str)
    def __on_uninstall_worker_result(self, rgame: RareGame, success: bool, message: str):
        if success:
            rgame.set_installed(False)
        else:
            QMessageBox.warning(None, self.tr("Uninstall - {}").format(rgame.app_title), message, QMessageBox.Close)
        rgame.state = RareGame.State.IDLE



from logging import getLogger
from typing import List

from PySide6.QtCore import Signal, Slot, QSettings
from PySide6.QtGui import QIcon, QAction
from PySide6.QtWidgets import QSystemTrayIcon, QMenu, QApplication

from rare.models.options import options
from rare.shared import RareCore

logger = getLogger("TrayIcon")


class TrayIcon(QSystemTrayIcon):
    # none:
    show_app: Signal = Signal()
    # int: exit code
    exit_app: Signal = Signal(int)

    def __init__(self, parent=None):
        super(TrayIcon, self).__init__(parent=parent)
        self.__parent = parent
        self.rcore = RareCore.instance()
        self.core = RareCore.instance().core()
        self.settings = QSettings()

        self.setIcon(QIcon(":/images/Rare.png"))
        self.setVisible(True)
        self.setToolTip(QApplication.applicationName())

        self.menu = QMenu()

        self.show_action = QAction(QApplication.applicationName())
        self.show_action.triggered.connect(self.show_app)
        self.menu.addAction(self.show_action)

        self.menu.addSeparator()
        self.text_action = QAction("Quick launch")
        self.text_action.setEnabled(False)
        self.menu.addAction(self.text_action)

        # We need to reference this separator to add game actions before it
        self.separator = self.menu.addSeparator()
        self.exit_action = QAction(self.tr("Quit"))
        self.exit_action.triggered.connect(lambda: self.exit_app.emit(0))
        self.menu.addAction(self.exit_action)

        self.game_actions: List[QAction] = []
        self.update_actions()

        self.setContextMenu(self.menu)

        self.signals = RareCore.instance().signals()
        self.signals.game.uninstalled.connect(self.remove_button)
        self.signals.application.notify.connect(self.notify)
        self.signals.application.update_tray.connect(self.update_actions)

    def last_played(self) -> List:
        last_played = [game for game in self.rcore.games if (game.metadata and game.is_installed)]
        last_played.sort(key=lambda g: g.metadata.last_played, reverse=True)
        return last_played[:5]

    @Slot(str, str)
    def notify(self, title: str, body: str):
        if self.settings.value(*options.notification):
            self.showMessage(
                f"{title} - {QApplication.applicationName()}", body, QSystemTrayIcon.MessageIcon.Information, 4000
            )

    @Slot()
    def update_actions(self):
        for action in self.game_actions:
            action.deleteLater()
        self.game_actions.clear()
        for rgame in self.last_played():
            a = TrayAction(self.rcore, rgame.app_title, rgame.app_name)
            self.menu.insertAction(self.separator, a)
            self.game_actions.append(a)

    @Slot(str)
    def remove_button(self, app_name: str):
        if action := next((i for i in self.game_actions if i.property("app_name") == app_name), None):
            self.game_actions.remove(action)
            action.deleteLater()


class TrayAction(QAction):
    def __init__(self, rcore: RareCore, text: str, app_name: str):
        super(TrayAction, self).__init__(text=text)
        self.rcore = rcore
        self.app_name = app_name
        self.triggered.connect(self.__on_triggered)

    def __on_triggered(self):
        self.rcore.get_game(self.app_name).launch()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'details.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFormLayout, QFrame, QHBoxLayout,
    QLabel, QLayout, QPushButton, QSizePolicy,
    QTextBrowser, QVBoxLayout, QWidget)

class Ui_StoreDetailsWidget(object):
    def setupUi(self, StoreDetailsWidget):
        if not StoreDetailsWidget.objectName():
            StoreDetailsWidget.setObjectName(u"StoreDetailsWidget")
        StoreDetailsWidget.resize(630, 382)
        StoreDetailsWidget.setWindowTitle(u"StoreDetailsWidget")
        self.main_layout = QHBoxLayout(StoreDetailsWidget)
        self.main_layout.setObjectName(u"main_layout")
        self.left_layout = QVBoxLayout()
        self.left_layout.setObjectName(u"left_layout")
        self.back_button = QPushButton(StoreDetailsWidget)
        self.back_button.setObjectName(u"back_button")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.back_button.sizePolicy().hasHeightForWidth())
        self.back_button.setSizePolicy(sizePolicy)
        self.back_button.setText(u"")
        self.back_button.setIconSize(QSize(32, 32))
        self.back_button.setFlat(True)

        self.left_layout.addWidget(self.back_button)


        self.main_layout.addLayout(self.left_layout)

        self.right_layout = QVBoxLayout()
        self.right_layout.setObjectName(u"right_layout")
        self.details_layout = QFormLayout()
        self.details_layout.setObjectName(u"details_layout")
        self.details_layout.setSizeConstraint(QLayout.SetFixedSize)
        self.details_layout.setFieldGrowthPolicy(QFormLayout.AllNonFixedFieldsGrow)
        self.details_layout.setLabelAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)
        self.details_layout.setHorizontalSpacing(12)
        self.details_layout.setVerticalSpacing(12)
        self.details_layout.setContentsMargins(6, 6, 6, 6)
        self.title_label = QLabel(StoreDetailsWidget)
        self.title_label.setObjectName(u"title_label")
        font = QFont()
        font.setBold(True)
        self.title_label.setFont(font)
        self.title_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(0, QFormLayout.LabelRole, self.title_label)

        self.title = QLabel(StoreDetailsWidget)
        self.title.setObjectName(u"title")
        self.title.setText(u"title")
        self.title.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)

        self.details_layout.setWidget(0, QFormLayout.ItemRole.FieldRole, self.title)

        self.developer_label = QLabel(StoreDetailsWidget)
        self.developer_label.setObjectName(u"developer_label")
        self.developer_label.setFont(font)
        self.developer_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(1, QFormLayout.LabelRole, self.developer_label)

        self.developer = QLabel(StoreDetailsWidget)
        self.developer.setObjectName(u"developer")
        self.developer.setText(u"developer")
        self.developer.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)

        self.details_layout.setWidget(1, QFormLayout.ItemRole.FieldRole, self.developer)

        self.publisher_label = QLabel(StoreDetailsWidget)
        self.publisher_label.setObjectName(u"publisher_label")
        self.publisher_label.setFont(font)
        self.publisher_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(2, QFormLayout.LabelRole, self.publisher_label)

        self.publisher = QLabel(StoreDetailsWidget)
        self.publisher.setObjectName(u"publisher")
        self.publisher.setText(u"publisher")
        self.publisher.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)

        self.details_layout.setWidget(2, QFormLayout.ItemRole.FieldRole, self.publisher)

        self.status_label = QLabel(StoreDetailsWidget)
        self.status_label.setObjectName(u"status_label")
        self.status_label.setFont(font)
        self.status_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(3, QFormLayout.LabelRole, self.status_label)

        self.status = QLabel(StoreDetailsWidget)
        self.status.setObjectName(u"status")

        self.details_layout.setWidget(3, QFormLayout.ItemRole.FieldRole, self.status)

        self.price_label = QLabel(StoreDetailsWidget)
        self.price_label.setObjectName(u"price_label")
        self.price_label.setFont(font)
        self.price_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(4, QFormLayout.LabelRole, self.price_label)

        self.tags_label = QLabel(StoreDetailsWidget)
        self.tags_label.setObjectName(u"tags_label")
        self.tags_label.setFont(font)
        self.tags_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(5, QFormLayout.LabelRole, self.tags_label)

        self.tags = QLabel(StoreDetailsWidget)
        self.tags.setObjectName(u"tags")
        self.tags.setText(u"tags")

        self.details_layout.setWidget(5, QFormLayout.ItemRole.FieldRole, self.tags)

        self.social_links_label = QLabel(StoreDetailsWidget)
        self.social_links_label.setObjectName(u"social_links_label")
        self.social_links_label.setFont(font)
        self.social_links_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(6, QFormLayout.LabelRole, self.social_links_label)

        self.actions_label = QLabel(StoreDetailsWidget)
        self.actions_label.setObjectName(u"actions_label")
        self.actions_label.setFont(font)
        self.actions_label.setAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)

        self.details_layout.setWidget(7, QFormLayout.LabelRole, self.actions_label)

        self.social_links = QWidget(StoreDetailsWidget)
        self.social_links.setObjectName(u"social_links")
        self.social_links_layout = QHBoxLayout(self.social_links)
        self.social_links_layout.setObjectName(u"social_links_layout")
        self.social_links_layout.setSizeConstraint(QLayout.SetFixedSize)
        self.social_links_layout.setContentsMargins(0, 0, 0, 0)

        self.details_layout.setWidget(6, QFormLayout.ItemRole.FieldRole, self.social_links)

        self.actions = QWidget(StoreDetailsWidget)
        self.actions.setObjectName(u"actions")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.actions.sizePolicy().hasHeightForWidth())
        self.actions.setSizePolicy(sizePolicy1)
        self.actions.setMinimumSize(QSize(250, 0))
        self.actions_layout = QVBoxLayout(self.actions)
        self.actions_layout.setObjectName(u"actions_layout")
        self.actions_layout.setContentsMargins(0, 0, 0, 0)
        self.store_button = QPushButton(self.actions)
        self.store_button.setObjectName(u"store_button")

        self.actions_layout.addWidget(self.store_button)

        self.wishlist_button = QPushButton(self.actions)
        self.wishlist_button.setObjectName(u"wishlist_button")

        self.actions_layout.addWidget(self.wishlist_button)


        self.details_layout.setWidget(7, QFormLayout.ItemRole.FieldRole, self.actions)

        self.price = QWidget(StoreDetailsWidget)
        self.price.setObjectName(u"price")
        sizePolicy1.setHeightForWidth(self.price.sizePolicy().hasHeightForWidth())
        self.price.setSizePolicy(sizePolicy1)
        self.price_layout = QHBoxLayout(self.price)
        self.price_layout.setObjectName(u"price_layout")
        self.price_layout.setSizeConstraint(QLayout.SetFixedSize)
        self.price_layout.setContentsMargins(0, 0, 0, 0)
        self.original_price = QLabel(self.price)
        self.original_price.setObjectName(u"original_price")
        self.original_price.setText(u"orignal")
        self.original_price.setTextInteractionFlags(Qt.LinksAccessibleByMouse|Qt.TextSelectableByMouse)

        self.price_layout.addWidget(self.original_price)

        self.discount_price = QLabel(self.price)
        self.discount_price.setObjectName(u"discount_price")
        self.discount_price.setText(u"discount")

        self.price_layout.addWidget(self.discount_price)


        self.details_layout.setWidget(4, QFormLayout.ItemRole.FieldRole, self.price)


        self.right_layout.addLayout(self.details_layout)

        self.requirements_frame = QFrame(StoreDetailsWidget)
        self.requirements_frame.setObjectName(u"requirements_frame")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.requirements_frame.sizePolicy().hasHeightForWidth())
        self.requirements_frame.setSizePolicy(sizePolicy2)
        self.requirements_frame.setFrameShape(QFrame.Shape.StyledPanel)
        self.requirements_frame.setFrameShadow(QFrame.Sunken)
        self.requirements_layout = QHBoxLayout(self.requirements_frame)
        self.requirements_layout.setObjectName(u"requirements_layout")
        self.requirements_layout.setContentsMargins(0, 0, 0, 0)

        self.right_layout.addWidget(self.requirements_frame)

        self.description_label = QTextBrowser(StoreDetailsWidget)
        self.description_label.setObjectName(u"description_label")
        self.description_label.setOpenExternalLinks(True)

        self.right_layout.addWidget(self.description_label)


        self.main_layout.addLayout(self.right_layout)

        self.main_layout.setStretch(1, 1)

        self.retranslateUi(StoreDetailsWidget)
    # setupUi

    def retranslateUi(self, StoreDetailsWidget):
        self.title_label.setText(QCoreApplication.translate("StoreDetailsWidget", u"Title", None))
        self.developer_label.setText(QCoreApplication.translate("StoreDetailsWidget", u"Developer", None))
        self.publisher_label.setText(QCoreApplication.translate("StoreDetailsWidget", u"Publisher", None))
        self.status_label.setText(QCoreApplication.translate("StoreDetailsWidget", u"Status", None))
        self.status.setText(QCoreApplication.translate("StoreDetailsWidget", u"You already own this game", None))
        self.price_label.setText(QCoreApplication.translate("StoreDetailsWidget", u"Price", None))
        self.tags_label.setText(QCoreApplication.translate("StoreDetailsWidget", u"Tags", None))
        self.social_links_label.setText(QCoreApplication.translate("StoreDetailsWidget", u"Links", None))
        self.actions_label.setText(QCoreApplication.translate("StoreDetailsWidget", u"Actions", None))
        self.store_button.setText(QCoreApplication.translate("StoreDetailsWidget", u"Buy in Epic Games Store", None))
        self.wishlist_button.setText(QCoreApplication.translate("StoreDetailsWidget", u"Add to wishlist", None))
        pass
    # retranslateUi




from PySide6.QtCore import Qt
from PySide6.QtWidgets import QTreeView, QDialog, QVBoxLayout

from rare.utils.json_formatter import QJsonModel


class DebugView(QTreeView):
    def __init__(self, data, parent=None):
        super(DebugView, self).__init__(parent=parent)
        self.setColumnWidth(0, 300)
        self.setWordWrap(True)
        self.model = QJsonModel(self)
        self.setModel(self.model)
        self.setContextMenuPolicy(Qt.ContextMenuPolicy.ActionsContextMenu)
        try:
            self.model.load(data)
        except Exception as e:
            pass
        self.resizeColumnToContents(0)


class DebugDialog(QDialog):
    def __init__(self, data, parent=None):
        super().__init__(parent=parent)
        self.resize(800, 600)

        layout = QVBoxLayout(self)
        view = DebugView(data, self)
        layout.addWidget(view)



import os
import platform
import time
from configparser import ConfigParser
from logging import getLogger
from typing import Union, Iterable, List, Tuple, Dict

from PySide6.QtCore import Signal, QObject

from rare.lgndr.core import LegendaryCore
from rare.models.game import RareGame
from rare.models.pathspec import PathSpec
from rare.shared.wrappers import Wrappers
from rare.utils import config_helper as config
from rare.utils.misc import path_size, format_size
from .worker import Worker

if platform.system() == "Windows":
    # noinspection PyUnresolvedReferences
    import winreg  # pylint: disable=E0401
    from legendary.lfs import windows_helpers
else:
    from rare.utils.compat import utils as compat_utils, steam

logger = getLogger("WineResolver")


class WinePathResolver(Worker):
    class Signals(QObject):
        result_ready = Signal(str, str)

    def __init__(self, core: LegendaryCore, app_name: str, path: str):
        super(WinePathResolver, self). __init__()
        self.signals = WinePathResolver.Signals()
        self.core = core
        self.app_name = app_name
        self.path = path

    @staticmethod
    def _configure_process(core: LegendaryCore, app_name: str) -> Tuple[List, Dict]:
        tool: steam.CompatibilityTool = None

        if config.get_boolean(app_name, "no_wine"):
            wrappers = Wrappers()
            for w in wrappers.get_game_wrapper_list(app_name):
                if w.is_compat_tool:
                    for t in steam.find_tools():
                        if t.checksum == w.checksum:
                            tool = t
                            break

        cmd = core.get_app_launch_command(
            app_name,
            wrapper=tool.as_str(steam.SteamVerb.RUN_IN_PREFIX) if tool is not None else None,
            disable_wine=config.get_boolean(app_name, "no_wine")
        )
        env = core.get_app_environment(app_name, disable_wine=config.get_boolean(app_name, "no_wine"))
        # pylint: disable=E0606
        env = compat_utils.get_host_environment(env, silent=True)

        return cmd, env

    @staticmethod
    def _resolve_unix_path(cmd, env, path: str) -> str:
        logger.info("Resolving path '%s'", path)
        wine_path = compat_utils.resolve_path(cmd, env, path)
        logger.info("Resolved Wine path '%s'", wine_path)
        unix_path = compat_utils.convert_to_unix_path(cmd, env, wine_path)
        logger.info("Resolved Unix path '%s'", unix_path)
        return unix_path

    def run_real(self):
        command, environ = self._configure_process(self.core, self.app_name)
        if not (command and environ):
            logger.error("Cannot setup %s, missing infomation", {type(self).__name__})
            self.signals.result_ready.emit("", self.app_name)

        path = self._resolve_unix_path(command, environ, self.path)
        self.signals.result_ready.emit(path, self.app_name)
        return


class WineSavePathResolver(WinePathResolver):

    def __init__(self, core: LegendaryCore, rgame: RareGame):
        path = PathSpec(core, rgame.igame).resolve_egl_path_vars(rgame.raw_save_path)
        super(WineSavePathResolver, self).__init__(rgame.core, rgame.app_name, str(path))
        self.rgame = rgame

    def run_real(self):
        logger.info("Resolving save path for %s (%s)", self.rgame.app_title, self.rgame.app_name)
        command, environ = self._configure_process(self.core, self.rgame.app_name)
        if not (command and environ):
            logger.error("Cannot setup %s, missing infomation", {type(self).__name__})
            self.signals.result_ready.emit("", self.rgame.app_name)

        path = self._resolve_unix_path(command, environ, self.path)
        # Clean wine output
        # pylint: disable=E1136
        if os.path.exists(path):
            self.rgame.save_path = path
        self.signals.result_ready.emit(path, self.rgame.app_name)
        return


class OriginWineWorker(WinePathResolver):
    def __init__(self, core: LegendaryCore, games: Union[Iterable[RareGame], RareGame]):
        super(OriginWineWorker, self).__init__(core, "", "")
        self.__cache: dict[str, ConfigParser] = {}
        self.core = core
        self.games = [games] if isinstance(games, RareGame) else games

    def run_real(self) -> None:
        t = time.time()

        for rgame in self.games:

            reg_path: str = rgame.game.metadata \
                .get("customAttributes", {}) \
                .get("RegistryPath", {}).get("value", None)
            if not reg_path:
                continue

            reg_key: str = rgame.game.metadata \
                .get("customAttributes", {}) \
                .get("RegistryKey", {}).get("value", None)
            if not reg_key:
                continue

            if platform.system() == "Windows":
                install_dir = windows_helpers.query_registry_value(winreg.HKEY_LOCAL_MACHINE, reg_path, reg_key)
            else:
                command, environ = self._configure_process(self.core, rgame.app_name)

                prefix = config.get_prefix(rgame.app_name)
                if not prefix:
                    return

                use_wine = True
                if not use_wine:
                    # lk: this is the original way of getting the path by parsing "system.reg"
                    reg = self.__cache.get(prefix, None) or compat_utils.read_registry("system.reg", prefix)
                    self.__cache[prefix] = reg

                    reg_path = reg_path.replace("SOFTWARE", "Software").replace("WOW6432Node", "Wow6432Node")
                    # lk: split and rejoin the registry path to avoid slash expansion
                    reg_path = "\\\\".join([x for x in reg_path.split("\\") if bool(x)])

                    install_dir = reg.get(reg_path, f'"{reg_key}"', fallback=None)
                else:
                    # lk: this is the alternative way of getting the path by using wine itself
                    install_dir = compat_utils.query_reg_key(command, environ, f"HKLM\\{reg_path}", reg_key)

                if install_dir:
                    logger.debug("Found Wine install directory %s", install_dir)
                    install_dir = compat_utils.convert_to_unix_path(command, environ, install_dir)
                    if install_dir:
                        logger.debug("Found Unix install directory %s", install_dir)
                    else:
                        logger.info("Could not find Unix install directory for '%s'", rgame.app_title)
                else:
                    logger.info("Could not find Wine install directory for '%s'", rgame.app_title)

            if install_dir:
                if os.path.isdir(install_dir):
                    install_size = path_size(install_dir)
                    rgame.set_origin_attributes(install_dir, install_size)
                    logger.info("Origin game '%s' (%s, %s)", rgame.app_title, install_dir, format_size(install_size))
                else:
                    logger.warning("Origin game '%s' (%s does not exist)", rgame.app_title, install_dir)
            else:
                logger.info("Origin game '%s' is not installed", rgame.app_title)
        logger.info("Origin worker finished in %ss", time.time() - t)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'about.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFormLayout, QLabel, QPushButton,
    QSizePolicy, QWidget)

class Ui_About(object):
    def setupUi(self, About):
        if not About.objectName():
            About.setObjectName(u"About")
        About.resize(507, 210)
        self.about_layout = QFormLayout(About)
        self.about_layout.setObjectName(u"about_layout")
        self.about_layout.setLabelAlignment(Qt.AlignmentFlag.AlignRight|Qt.AlignTrailing|Qt.AlignmentFlag.AlignVCenter)
        self.version_label = QLabel(About)
        self.version_label.setObjectName(u"version_label")
        font = QFont()
        font.setBold(True)
        self.version_label.setFont(font)

        self.about_layout.setWidget(0, QFormLayout.LabelRole, self.version_label)

        self.version = QLabel(About)
        self.version.setObjectName(u"version")
        self.version.setText(u"error")

        self.about_layout.setWidget(0, QFormLayout.ItemRole.FieldRole, self.version)

        self.update_label = QLabel(About)
        self.update_label.setObjectName(u"update_label")
        self.update_label.setFont(font)

        self.about_layout.setWidget(1, QFormLayout.LabelRole, self.update_label)

        self.update_lbl = QLabel(About)
        self.update_lbl.setObjectName(u"update_lbl")
        self.update_lbl.setText(u"error")

        self.about_layout.setWidget(1, QFormLayout.ItemRole.FieldRole, self.update_lbl)

        self.open_browser = QPushButton(About)
        self.open_browser.setObjectName(u"open_browser")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.open_browser.sizePolicy().hasHeightForWidth())
        self.open_browser.setSizePolicy(sizePolicy)

        self.about_layout.setWidget(2, QFormLayout.ItemRole.FieldRole, self.open_browser)

        self.dev_label = QLabel(About)
        self.dev_label.setObjectName(u"dev_label")
        self.dev_label.setFont(font)

        self.about_layout.setWidget(3, QFormLayout.LabelRole, self.dev_label)

        self.dev = QLabel(About)
        self.dev.setObjectName(u"dev")
        self.dev.setText(u"<a href='https://github.com/Dummerle' style='color: #2980b9; text-decoration:none'>Dummerle</a>\n"
"<br>\n"
"<a href='https://github.com/loathingkernel' style='color: #2980b9; text-decoration:none'>loathingkernel</a>")
        self.dev.setOpenExternalLinks(True)

        self.about_layout.setWidget(3, QFormLayout.ItemRole.FieldRole, self.dev)

        self.lgd_dev_label = QLabel(About)
        self.lgd_dev_label.setObjectName(u"lgd_dev_label")
        self.lgd_dev_label.setFont(font)

        self.about_layout.setWidget(5, QFormLayout.LabelRole, self.lgd_dev_label)

        self.lgd_dev = QLabel(About)
        self.lgd_dev.setObjectName(u"lgd_dev")
        self.lgd_dev.setText(u"<a href='https://github.com/derrod/' style='color: #2980b9; text-decoration:none'>derrod</a>")
        self.lgd_dev.setOpenExternalLinks(True)

        self.about_layout.setWidget(5, QFormLayout.ItemRole.FieldRole, self.lgd_dev)

        self.license_label = QLabel(About)
        self.license_label.setObjectName(u"license_label")
        self.license_label.setFont(font)

        self.about_layout.setWidget(6, QFormLayout.LabelRole, self.license_label)

        self.license = QLabel(About)
        self.license.setObjectName(u"license")

        self.about_layout.setWidget(6, QFormLayout.ItemRole.FieldRole, self.license)

        self.contrib_label = QLabel(About)
        self.contrib_label.setObjectName(u"contrib_label")
        self.contrib_label.setFont(font)

        self.about_layout.setWidget(4, QFormLayout.LabelRole, self.contrib_label)

        self.label = QLabel(About)
        self.label.setObjectName(u"label")
        self.label.setText(u"<a href='https://github.com/MultisampledNight' style='color: #2980b9; text-decoration:none'>MultisampledNight</a> <i>artwork, bug hunting, testing</i>\n"
"<br>\n"
"<a href='https://github.com/invertedEcho' style='color: #2980b9; text-decoration:none'>invertedEcho</a> <i>moving games, environment variables</i>\n"
"<br>\n"
"<a href='https://github.com/ChemicalXandco' style='color: #2980b9; text-decoration:none'>ChemicalXandco</a> <i>downloads improvements, windows integration</i>\n"
"<br>\n"
"<a href='https://github.com/gnanini' style='color: #2980b9; text-decoration:none'>gnanini</a> <i>steam grades</i>")

        self.about_layout.setWidget(4, QFormLayout.ItemRole.FieldRole, self.label)


        self.retranslateUi(About)
    # setupUi

    def retranslateUi(self, About):
        About.setWindowTitle(QCoreApplication.translate("About", u"About", None))
        self.version_label.setText(QCoreApplication.translate("About", u"Version", None))
        self.update_label.setText(QCoreApplication.translate("About", u"Update", None))
        self.open_browser.setText(QCoreApplication.translate("About", u"Download latest release", None))
        self.dev_label.setText(QCoreApplication.translate("About", u"Rare Developers", None))
#if QT_CONFIG(tooltip)
        self.dev.setToolTip(QCoreApplication.translate("About", u"Github", None))
#endif // QT_CONFIG(tooltip)
        self.lgd_dev_label.setText(QCoreApplication.translate("About", u"Legendary Developer", None))
#if QT_CONFIG(tooltip)
        self.lgd_dev.setToolTip(QCoreApplication.translate("About", u"Github", None))
#endif // QT_CONFIG(tooltip)
        self.license_label.setText(QCoreApplication.translate("About", u"License", None))
        self.license.setText(QCoreApplication.translate("About", u"GNU General Public License v3.0", None))
        self.contrib_label.setText(QCoreApplication.translate("About", u"Rare Contributors", None))
    # retranslateUi




from typing import Optional, List

from PySide6.QtCore import Qt, Slot, Signal
from PySide6.QtGui import QShowEvent
from PySide6.QtWidgets import QFrame, QMessageBox, QToolBox

from rare.models.game import RareGame
from rare.shared import LegendaryCoreSingleton, GlobalSignalsSingleton
from rare.ui.components.tabs.games.game_info.dlcs import Ui_GameDlcs
from rare.ui.components.tabs.games.game_info.dlc_widget import Ui_GameDlcWidget
from rare.widgets.image_widget import ImageWidget, ImageSize
from rare.widgets.side_tab import SideTabContents
from rare.utils.misc import widget_object_name, qta_icon


class GameDlcWidget(QFrame):
    def __init__(self, rgame: RareGame, rdlc: RareGame, parent=None):
        super(GameDlcWidget, self).__init__(parent=parent)
        self.ui = Ui_GameDlcWidget()
        self.ui.setupUi(self)
        self.setObjectName(widget_object_name(self, rdlc.app_name))
        self.rgame = rgame
        self.rdlc = rdlc

        self.image = ImageWidget(self)
        self.image.setFixedSize(ImageSize.LibraryIcon)
        self.ui.dlc_layout.insertWidget(0, self.image)

        self.ui.dlc_name.setText(rdlc.app_title)
        self.ui.version.setText(rdlc.version)
        self.ui.app_name.setText(rdlc.app_name)

        # self.image.setPixmap(rdlc.get_pixmap_icon(rdlc.is_installed))
        self.__update()

        rdlc.signals.widget.update.connect(self.__update)

    @Slot()
    def __update(self):
        self.ui.action_button.setEnabled(self.rdlc.is_idle)
        self.image.setPixmap(self.rdlc.get_pixmap(ImageSize.LibraryIcon, self.rdlc.is_installed))

    def showEvent(self, a0: QShowEvent) -> None:
        if a0.spontaneous():
            return super().showEvent(a0)
        if not self.rdlc.has_pixmap:
            self.rdlc.load_pixmaps()
        super().showEvent(a0)


class InstalledGameDlcWidget(GameDlcWidget):
    uninstalled = Signal(RareGame)

    def __init__(self, rgame: RareGame, rdlc: RareGame, parent=None):
        super(InstalledGameDlcWidget, self).__init__(rgame=rgame, rdlc=rdlc, parent=parent)
        # lk: set object names for CSS properties
        self.ui.action_button.setObjectName("UninstallButton")
        self.ui.action_button.clicked.connect(self.uninstall_dlc)
        self.ui.action_button.setText(self.tr("Uninstall DLC"))
        self.ui.action_button.setIcon(qta_icon("ri.uninstall-line"))
        # lk: don't reference `self.rdlc` here because the object has been deleted
        rdlc.signals.game.uninstalled.connect(self.__uninstalled)

    @Slot()
    def __uninstalled(self):
        self.uninstalled.emit(self.rdlc)

    def uninstall_dlc(self):
        self.rdlc.uninstall()


class AvailableGameDlcWidget(GameDlcWidget):
    installed = Signal(RareGame)

    def __init__(self, rgame: RareGame, rdlc: RareGame, parent=None):
        super(AvailableGameDlcWidget, self).__init__(rgame=rgame, rdlc=rdlc, parent=parent)
        # lk: set object names for CSS properties
        self.ui.action_button.setObjectName("InstallButton")
        self.ui.action_button.clicked.connect(self.install_dlc)
        self.ui.action_button.setText(self.tr("Install DLC"))
        self.ui.action_button.setIcon(qta_icon("ri.install-line"))

        # lk: don't reference `self.rdlc` here because the object has been deleted
        rdlc.signals.game.installed.connect(self.__installed)

    @Slot()
    def __installed(self):
        self.installed.emit(self.rdlc)

    def install_dlc(self):
        if not self.rgame.is_installed:
            QMessageBox.warning(
                self,
                self.tr("Error"),
                self.tr("Base Game is not installed. Please install {} first").format(self.rgame.app_title),
            )
            return
        self.rdlc.install()


class GameDlcs(QToolBox, SideTabContents):

    def __init__(self, parent=None):
        super(GameDlcs, self).__init__(parent=parent)
        self.implements_scrollarea = True
        self.ui = Ui_GameDlcs()
        self.ui.setupUi(self)
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()

        self.rgame: Optional[RareGame] = None

    def list_installed(self) -> List[InstalledGameDlcWidget]:
        return self.ui.installed_dlc_container.findChildren(InstalledGameDlcWidget, options=Qt.FindChildOption.FindDirectChildrenOnly)

    def list_available(self) -> List[AvailableGameDlcWidget]:
        return self.ui.available_dlc_container.findChildren(AvailableGameDlcWidget, options=Qt.FindChildOption.FindDirectChildrenOnly)

    def get_installed(self, app_name: str) -> Optional[InstalledGameDlcWidget]:
        return self.ui.installed_dlc_container.findChild(
            InstalledGameDlcWidget,
            name=widget_object_name(InstalledGameDlcWidget, app_name),
            options=Qt.FindChildOption.FindDirectChildrenOnly
        )

    def get_available(self, app_name: str) -> Optional[AvailableGameDlcWidget]:
        return self.ui.available_dlc_container.findChild(
            AvailableGameDlcWidget,
            name=widget_object_name(AvailableGameDlcWidget, app_name),
            options=Qt.FindChildOption.FindDirectChildrenOnly
        )

    def update_installed_page(self):
        have_installed = bool(self.list_installed())
        self.ui.installed_dlc_label.setVisible(not have_installed)
        self.ui.installed_dlc_container.setVisible(have_installed)
        if not have_installed:
            self.setCurrentWidget(self.ui.available_dlc_page)

    def update_available_page(self):
        have_available = bool(self.list_available())
        self.ui.available_dlc_label.setVisible(not have_available)
        self.ui.available_dlc_container.setVisible(have_available)
        if not have_available:
            self.setCurrentWidget(self.ui.installed_dlc_page)

    def append_installed(self, rdlc: RareGame):
        self.ui.installed_dlc_label.setVisible(False)
        self.ui.installed_dlc_container.setVisible(True)
        a_widget: AvailableGameDlcWidget = self.get_available(rdlc.app_name)
        if a_widget is not None:
            self.ui.available_dlc_container.layout().removeWidget(a_widget)
            a_widget.deleteLater()
        i_widget: InstalledGameDlcWidget = InstalledGameDlcWidget(
            self.rgame, rdlc, self.ui.installed_dlc_container
        )
        i_widget.destroyed.connect(self.update_installed_page)
        i_widget.uninstalled.connect(self.append_available)
        self.ui.installed_dlc_container.layout().addWidget(i_widget)

    def append_available(self, rdlc: RareGame):
        self.ui.available_dlc_label.setVisible(False)
        self.ui.available_dlc_container.setVisible(True)
        i_widget: InstalledGameDlcWidget = self.get_installed(rdlc.app_name)
        if i_widget is not None:
            self.ui.available_dlc_container.layout().removeWidget(i_widget)
            i_widget.deleteLater()
        a_widget: AvailableGameDlcWidget = AvailableGameDlcWidget(
            self.rgame, rdlc, self.ui.available_dlc_container
        )
        a_widget.destroyed.connect(self.update_available_page)
        a_widget.installed.connect(self.append_installed)
        self.ui.available_dlc_container.layout().addWidget(a_widget)

    def update_dlcs(self, rgame: RareGame):
        self.rgame = rgame
        self.set_title.emit(self.rgame.app_title)

        for i_widget in self.list_installed():
            self.ui.installed_dlc_container.layout().removeWidget(i_widget)
            i_widget.deleteLater()

        for a_widget in self.list_available():
            self.ui.available_dlc_container.layout().removeWidget(a_widget)
            a_widget.deleteLater()

        for dlc in sorted(self.rgame.owned_dlcs, key=lambda x: x.app_title):
            if dlc.is_installed:
                self.append_installed(rdlc=dlc)
            else:
                self.append_available(rdlc=dlc)

        if not self.list_available():
            self.setCurrentWidget(self.ui.installed_dlc_page)
        if not self.list_installed():
            self.setCurrentWidget(self.ui.available_dlc_page)



from logging import getLogger

from PySide6.QtCore import QSettings, Slot
from PySide6.QtGui import QShowEvent
from PySide6.QtWidgets import QStackedWidget, QVBoxLayout, QWidget, QScrollArea, QFrame

from rare.models.game import RareGame
from rare.shared import (
    LegendaryCoreSingleton,
    GlobalSignalsSingleton,
    ArgumentsSingleton,
    ImageManagerSingleton,
)
from rare.shared import RareCore
from rare.models.options import options
from .details import GameInfoTabs
from .widgets import LibraryWidgetController, LibraryFilter, LibraryOrder, LibraryView
from .head_bar import LibraryHeadBar
from .integrations import IntegrationsTabs

logger = getLogger("GamesLibrary")


class GamesLibrary(QStackedWidget):
    def __init__(self, parent=None):
        super(GamesLibrary, self).__init__(parent=parent)
        self.rcore = RareCore.instance()
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()
        self.args = ArgumentsSingleton()
        self.image_manager = ImageManagerSingleton()
        self.settings = QSettings()

        self.games_page = QWidget(parent=self)
        games_page_layout = QVBoxLayout(self.games_page)
        self.addWidget(self.games_page)

        self.head_bar = LibraryHeadBar(parent=self.games_page)
        self.head_bar.goto_import.connect(self.show_import)
        self.head_bar.goto_egl_sync.connect(self.show_egl_sync)
        self.head_bar.goto_eos_ubisoft.connect(self.show_eos_ubisoft)
        games_page_layout.addWidget(self.head_bar)

        self.game_info_page = GameInfoTabs(self)
        self.game_info_page.back_clicked.connect(lambda: self.setCurrentWidget(self.games_page))
        self.game_info_page.import_clicked.connect(self.show_import)
        self.addWidget(self.game_info_page)

        self.integrations_page = IntegrationsTabs(self)
        self.integrations_page.back_clicked.connect(lambda: self.setCurrentWidget(self.games_page))
        self.addWidget(self.integrations_page)

        self.view_scroll = QScrollArea(self.games_page)
        self.view_scroll.setWidgetResizable(True)
        self.view_scroll.setFrameShape(QFrame.Shape.StyledPanel)
        self.view_scroll.horizontalScrollBar().setDisabled(True)

        library_view = LibraryView(self.settings.value(*options.library_view))
        self.library_controller = LibraryWidgetController(library_view, self.view_scroll)
        games_page_layout.addWidget(self.view_scroll)

        self.head_bar.search_bar.textChanged.connect(self.search_games)
        self.head_bar.search_bar.textChanged.connect(self.scroll_to_top)
        self.head_bar.filterChanged.connect(self.filter_games)
        self.head_bar.filterChanged.connect(self.scroll_to_top)
        self.head_bar.orderChanged.connect(self.order_games)
        self.head_bar.orderChanged.connect(self.scroll_to_top)
        self.head_bar.refresh_list.clicked.connect(self.library_controller.update_game_views)

        # signals
        self.signals.game.installed.connect(self.update_count_games_label)
        self.signals.game.uninstalled.connect(self.update_count_games_label)

        self.init = True

    def showEvent(self, a0: QShowEvent):
        if a0.spontaneous() or not self.init:
            return super().showEvent(a0)
        self.setup_game_list()
        self.init = False
        return super().showEvent(a0)

    @Slot()
    def scroll_to_top(self):
        self.view_scroll.verticalScrollBar().setSliderPosition(
            self.view_scroll.verticalScrollBar().minimum()
        )

    @Slot()
    @Slot(str)
    def show_import(self, app_name: str = None):
        self.setCurrentWidget(self.integrations_page)
        self.integrations_page.show_import(app_name)

    @Slot()
    def show_egl_sync(self):
        self.setCurrentWidget(self.integrations_page)
        self.integrations_page.show_egl_sync()

    @Slot()
    def show_eos_ubisoft(self):
        self.setCurrentWidget(self.integrations_page)
        self.integrations_page.show_eos_ubisoft()

    @Slot(RareGame)
    def show_game_info(self, rgame):
        self.game_info_page.update_game(rgame)
        self.setCurrentWidget(self.game_info_page)

    @Slot()
    def update_count_games_label(self):
        self.head_bar.set_games_count(
            len([game for game in self.rcore.games if game.is_installed]),
            len(list(self.rcore.games)),
        )

    def setup_game_list(self):
        for rgame in self.rcore.games:
            widget = self.add_library_widget(rgame)
            if not widget:
                logger.warning("Excluding %s from the game list", rgame.app_title)
                continue
        self.filter_games(self.head_bar.current_filter())
        self.order_games(self.head_bar.current_order())
        self.update_count_games_label()

    def add_library_widget(self, rgame: RareGame):
        try:
            widget = self.library_controller.add_game(rgame)
        except Exception as e:
            logger.error("Could not add widget for %s to library: %s", rgame.app_name, e)
            return None
        widget.show_info.connect(self.show_game_info)
        return widget

    @Slot(str)
    def search_games(self, search_text: str = ""):
        self.filter_games(self.head_bar.current_filter(), search_text)

    @Slot(object)
    @Slot(object, str)
    def filter_games(self, library_filter: LibraryFilter = LibraryFilter.ALL, search_text: str = ""):
        if not search_text and (t := self.head_bar.search_bar.text()):
            search_text = t

        self.library_controller.filter_game_views(library_filter, search_text.lower())

    @Slot(object)
    @Slot(object, str)
    def order_games(self, library_order: LibraryOrder = LibraryFilter.ALL, search_text: str = ""):
        if not search_text and (t := self.head_bar.search_bar.text()):
            search_text = t

        self.library_controller.order_game_views(library_order, search_text.lower())



import os
from typing import Callable, Optional, Set, Any, Tuple

from legendary.core import LegendaryCore
from legendary.models.config import LGDConf

_config: Optional[LGDConf] = None
_save_config: Optional[Callable[[], None]] = None


def init_config_handler(core: LegendaryCore):
    global _config, _save_config
    _config = core.lgd.config
    _save_config = core.lgd.save_config


def save_config():
    if _save_config is None:
        raise RuntimeError("Uninitialized use of config_helper")
    _save_config()


def set_option(app_name: str, option: str, value: str) -> None:
    value = value.replace("%%", "%").replace("%", "%%")
    if not _config.has_section(app_name):
        _config[app_name] = {}
    _config.set(app_name, option, value)
    # save_config()


def set_boolean(app_name: str, option: str, value: bool) -> None:
    set_option(app_name, option, str(value).lower())


def set_envvar(app_name: str, envvar: str, value: str) -> None:
    set_option(f"{app_name}.env", envvar, value)


def remove_section(app_name: str) -> None:
    return
    # Disabled due to env variables implementation
    # if _config.has_section(app_name):
    #     _config.remove_section(app_name)
    #     save_config()


def remove_option(app_name: str, option: str) -> None:
    if _config.has_option(app_name, option):
        _config.remove_option(app_name, option)
    # if _config.has_section(app_name) and not _config[app_name]:
    #     _config.remove_section(app_name)
    # save_config()


def remove_envvar(app_name: str, option: str) -> None:
    remove_option(f"{app_name}.env", option)


def save_option(app_name: str, option: str, value: str) -> None:
    if value:
        set_option(app_name, option, value)
    else:
        remove_option(app_name, option)


def get_option(app_name: str, option: str, fallback: Any = None) -> str:
    return _config.get(app_name, option, fallback=fallback)


def get_option_fallback(app_name: str, option: str, fallback: Any = None) -> str:
    _option = get_option("default", option, fallback=fallback)
    _option = get_option(app_name, option, fallback=_option)
    return _option


def get_boolean(app_name: str, option: str, fallback: bool = False) -> bool:
    return _config.getboolean(app_name, option, fallback=fallback)


def save_envvar(app_name: str, option: str, value: str) -> None:
    if value:
        set_envvar(app_name, option, value)
    else:
        remove_envvar(app_name, option)


def get_envvar(app_name: str, option: str, fallback: Any = None) -> str:
    return get_option(f"{app_name}.env", option, fallback=fallback)


def get_envvar_fallback(app_name: str, option: str, fallback: Any = None) -> str:
    _option = _config.get("default.env", option, fallback=fallback)
    _option = _config.get(f'{app_name}.env', option, fallback=_option)
    return _option


def save_wine_prefix(app_name: str, value: str) -> None:
    save_envvar(app_name, "WINEPREFIX", value)
    save_option(app_name, "wine_prefix", value)


def get_wine_prefix(app_name: str, fallback: Any = None):
    _prefix = get_envvar(app_name, 'WINEPREFIX', fallback=fallback)
    _prefix = get_option(app_name, 'wine_prefix', fallback=_prefix)
    return _prefix


def get_wine_prefix_fallback(app_name: str, fallback: Any = None) -> str:
    _prefix = get_wine_prefix("default", fallback)
    _prefix = get_wine_prefix(app_name, fallback=_prefix)
    return _prefix


def save_proton_compatdata(app_name: str, value: str) -> None:
    save_envvar(app_name, "STEAM_COMPAT_DATA_PATH", value)


def get_proton_compatdata(app_name: Optional[str] = None, fallback: Any = None) -> str:
    _compat = get_envvar(app_name, "STEAM_COMPAT_DATA_PATH", fallback=fallback)
    # return os.path.join(_compat, "pfx") if _compat else fallback
    return _compat


def get_proton_compatdata_fallback(app_name: Optional[str] = None, fallback: Any = None) -> str:
    _compat = get_envvar_fallback(app_name, "STEAM_COMPAT_DATA_PATH", fallback=fallback)
    # return os.path.join(_compat, "pfx") if _compat else fallback
    return _compat


def get_wine_prefixes() -> Set[Tuple[str, str]]:
    _prefixes: Set[Tuple[str, str]] = set()
    for name, section in _config.items():
        pfx = section.get("WINEPREFIX") or section.get("wine_prefix")
        if pfx:
            _prefixes.update([(pfx, name[:-len(".env")] if name.endswith(".env") else name)])
    _prefixes = {(os.path.expanduser(p), n) for p, n in _prefixes}
    return {(p, n) for p, n in _prefixes if os.path.isdir(p)}


def get_proton_prefixes() -> Set[Tuple[str, str]]:
    _prefixes: Set[Tuple[str, str]] = set()
    for name, section in _config.items():
        pfx = os.path.join(compatdata, "pfx") if (compatdata := section.get("STEAM_COMPAT_DATA_PATH")) else ""
        if pfx:
            _prefixes.update([(pfx, name[:-len(".env")] if name.endswith(".env") else name)])
    _prefixes = {(os.path.expanduser(p), n) for p, n in _prefixes}
    return {(p, n) for p, n in _prefixes if os.path.isdir(p)}


def get_prefixes() -> Set[Tuple[str, str]]:
    return get_wine_prefixes().union(get_proton_prefixes())


def prefix_exists(pfx: str) -> bool:
    return os.path.isdir(pfx) and os.path.isfile(os.path.join(pfx, "user.reg"))


def get_prefix(app_name: str = "default") -> Optional[str]:
    _compat_path = _config.get(f"{app_name}.env", "STEAM_COMPAT_DATA_PATH", fallback=None)
    if _compat_path and prefix_exists(_compat_prefix := os.path.join(_compat_path, "pfx")):
        return _compat_prefix

    _wine_prefix = _config.get(f"{app_name}.env", "WINEPREFIX", fallback=None)
    _wine_prefix = _config.get(app_name, "wine_prefix", fallback=_wine_prefix)
    if _wine_prefix and prefix_exists(_wine_prefix):
        return _wine_prefix

    _compat_path = _config.get("default.env", "STEAM_COMPAT_DATA_PATH", fallback=None)
    if _compat_path and prefix_exists(_compat_prefix := os.path.join(_compat_path, "pfx")):
        return _compat_prefix

    _wine_prefix = _config.get("default.env", "WINEPREFIX", fallback=None)
    _wine_prefix = _config.get("default", "wine_prefix", fallback=_wine_prefix)
    if _wine_prefix and prefix_exists(_wine_prefix):
        return _wine_prefix

    return None
